package ru.ispras.telecomtesk.TLS13server;

import javax.net.ssl.SSLParameters;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import java.io.*;

/*
 * Don't forget to set the following system properties when you run the class:
 *
 *     javax.net.ssl.keyStore
 *     javax.net.ssl.keyStorePassword
 *     javax.net.ssl.trustStore
 *     javax.net.ssl.trustStorePassword
 *
 * More details can be found in JSSE docs.
 *
 * For example:
 * 
 *     java -cp classes \
 *         -Djavax.net.ssl.keyStore=keystore \
 *         -Djavax.net.ssl.keyStorePassword=passphrase \
 *         -Djavax.net.ssl.trustStore=keystore \
 *         -Djavax.net.ssl.trustStorePassword=passphrase \
 *             TLS13server
 */


public class TLS13server 
{
    private static final String[] protocols = new String[] {"TLSv1.3","TLSv1.2","TLSv1.1"};
    private static final int serverPort = 12345; // in millis

	public static void main(String[] argv) throws Exception
	{
		System.setProperty("javax.net.debug", "all");
		TLSServer server = TLSServer.createSocket(serverPort);
		System.out.println("Start server");
		while(true) {
			step(server);
		}
	}

	private static void step(TLSServer server) throws IOException {
		SSLSocket sock = server.accept();
		System.out.println("Connection from " + sock.getRemoteSocketAddress());
		try 
		{
			InputStream input = sock.getInputStream();
			OutputStream out = sock.getOutputStream();
			byte[] msg = new byte[1500];

			sock.startHandshake();
			System.out.println("startHandshake complete");
			while(true)
			{
				int len = input.read(msg);
				if (len < 0)
				{
					return;
				}
				if (len > 0)
				{
					System.out.println("Sending " + len + " bytes to " + sock.getRemoteSocketAddress());
					out.write(msg, 0, len);
				}
			}
		}
		catch (IOException e) {
			System.out.println("Thread Step: " + e);
		}
		finally
		{
			System.out.println("Connection from " + sock.getRemoteSocketAddress() + " closed");
			sock.close();
		}
	}
	
	public static class TLSServer implements AutoCloseable {
		private final SSLServerSocket sock;
	
		private TLSServer(SSLServerSocket sock) {
			this.sock = sock;
		}
	
		public static TLSServer createSocket(int port) throws IOException 
		{
            SSLServerSocket socket = (SSLServerSocket)
                    SSLServerSocketFactory.getDefault().createServerSocket(port);
            socket.setEnabledProtocols(protocols);
            return new TLSServer(socket);
		}
	
		public SSLSocket accept() throws IOException
		{
			SSLSocket s = (SSLSocket) sock.accept();
			return s;
		}

		@Override
    	public void close() throws IOException {
        	if (sock != null && !sock.isClosed()) {
        	    sock.close();
        	}
    	}
	}
}
