import java.io.*;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketAddress;
import java.security.GeneralSecurityException;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
 
import javax.net.ssl.KeyManager;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.X509KeyManager; 
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLSocket;


public class TLSclient {
	
	private static SSLContext sc = null;
	private static SSLSocketFactory sslsocketfactory = null;
	
	private static KeyManager[] getKeyManagerArray(String keystore,
			String keystorePasswd) {
		KeyManager[] ret = null;
		String keyFile = "keystore";
 
		if (null != keystore) {
			keyFile = keystore;
		} else
			keyFile = "keystore.jks";
		
		File t = new File(keyFile);
		if (!t.exists()) {
			throw new RuntimeException("Could not find key manager file");
		}
		
		if (null == keystorePasswd) {
			keystorePasswd = "123456";
		}
		try {
			System.out.println("Using keystore: " + keyFile);
			KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509", "SunJSSE");
			KeyStore ks = KeyStore.getInstance("JKS");
			// initialize KeyStore object using keystore name
			ks.load(new FileInputStream(keyFile), keystorePasswd.toCharArray());
			kmf.init(ks, keystorePasswd.toCharArray());
			ret = kmf.getKeyManagers();
		} catch (Exception e) {
			System.out.println("getKeyManagerArray Exception");
			e.printStackTrace();
		}
 
		return ret;
	}
	
	private static SSLContext getSSLContext() throws NoSuchAlgorithmException, KeyManagementException {
		SSLContext ret = SSLContext.getInstance("TLSv1.3");
//		SSLContext ret = SSLContext.getInstance("TLSv1.2");
//		SSLContext ret = SSLContext.getInstance("TLSv1");
		//		KeyManager[] km = getKeyManagerArray(null, null);
//		KeyManager[] km = getKeyManagerArray("keystoreserver.jks", "serverpwd");
//		KeyManager[] km = getKeyManagerArray("keystoreclient.jks", "clientpwd");
		KeyManager[] km = getKeyManagerArray("keystore", "secret");
		ret.init(km, null, null);
		return ret;
	}
	
	public static void main(String[] arstring) throws GeneralSecurityException,  IOException {

		ServerSocket serversocket = null;
		
		System.setProperty("javax.net.debug", "all");
		java.lang.System.setProperty("sun.security.ssl.allowUnsafeRenegotiation", "true");

		try {
//			sc = getSSLContext();
			serversocket = new ServerSocket();
			serversocket.setReuseAddress(true);
			serversocket.bind(new InetSocketAddress("127.0.0.2",5555));
			System.out.println("Create agent socket 127.0.0.2:5555");

			sslsocketfactory = (SSLSocketFactory) SSLSocketFactory.getDefault();
//			sslsocketfactory = (SSLSocketFactory) sc.getSocketFactory();

			while(true)
			{
				waitclient(serversocket);
			}
		}
		catch (Exception e) {
			System.out.println(e.getMessage());
        }
	}
		
	private static void waitclient(ServerSocket server) throws IOException {
		
		System.out.println("agent accept");
		Socket sock = server.accept();
		System.out.println("waitclient: Connection from " + sock.getRemoteSocketAddress());
		
		InputStream inputstream = sock.getInputStream();
		OutputStream outputstream = sock.getOutputStream();

		InputStreamReader inputstreamreader = new InputStreamReader(
				inputstream);
		BufferedReader bufferedreader = new BufferedReader(
				inputstreamreader);

		OutputStreamWriter outputstreamwriter = new OutputStreamWriter(
				outputstream);
		BufferedWriter bufferedwriter = new BufferedWriter(
				outputstreamwriter);
		System.out.println("Create input-output channels");

		try {
			String str = null;
			String dest_ip = null;
			String dest_port = null;
			boolean waitInputString = true;
			
			while (waitInputString) {
				str = bufferedreader.readLine();
				System.out.println(str);
				System.out.flush();
				if(str.startsWith("close")){
					System.out.println("waitclient: received close string");
					break;
				}
				if(!str.startsWith("start TLS to ")){
					System.out.println("waitclient: wait string: 'start TLS to <ip_address:port>'");
					System.out.println("waitclient: wait string: 'start TLS to <port>'");
					bufferedwriter.write("waitclient: wait string: 'start TLS to addr <ip_address:port>'");
					bufferedwriter.write("waitclient: wait string: 'start TLS to port <port>'");
					break;
				}
				String addr_dest = str.substring(13);
				int ind = addr_dest.indexOf(':');
				if(ind < 0) {
					dest_port = addr_dest;
				}
				else if(addr_dest.length() < ind + 2) {
					System.out.println("waitclient: wait string: 'start TLS to <ip_address:port>'");
					System.out.println("waitclient: wait string: 'start TLS to <port>'");
					bufferedwriter.write("waitclient: wait string: 'start TLS to addr <ip_address:port>'");
					bufferedwriter.write("waitclient: wait string: 'start TLS to port <port>'");
					break;
				}
				else {
					dest_ip = addr_dest.substring(0, ind);
					dest_port = addr_dest.substring(ind+1);
				}
				
				if((dest_ip == null) || (dest_ip.length() == 0)) {
					System.out.println("startTLSclient: server ip_address not defined, get remote socket address");
					dest_ip = sock.getInetAddress().getHostAddress();
				}
				startTLSclient(dest_ip,dest_port);
			}
			System.out.println("waitclient: Connection from " + sock.getRemoteSocketAddress() + " closed");
			outputstreamwriter.close();
			bufferedwriter.close();
			sock.close();
		} catch (Exception exception) {
			exception.printStackTrace();
		}
		
	}
	
	
	private static void startTLSclient(String serv_ip, String serv_port) throws IOException {
		
		if(serv_ip == null || serv_port == null) {
			System.out.println("startTLSclient: server ip_address or port not defined");
			return;
		}

		SSLSocket sslsocket = null;
		
		try 
		{
			System.out.println("startTLSclient 1: " + serv_ip + ":" + Integer.parseInt(serv_port));
			sslsocket = (SSLSocket) sslsocketfactory.createSocket(serv_ip,Integer.parseInt(serv_port));
			
			System.out.println("startTLSclient 2: SSLSocket created");
//			final String[] enabledCipherSuites = { "TLS_DHE_DSS_WITH_AES_128_CBC_SHA" };
//			sslsocket.setEnabledCipherSuites(enabledCipherSuites);
			String[] sp = sslsocket.getSupportedProtocols();
			String[] ss = sslsocket.getEnabledCipherSuites();
			sslsocket.setEnabledCipherSuites(ss);

			System.out.println("SupportedProtocols");
			for(int i=0;i<sp.length;i++){
				System.out.println(sp[i]);
			}

//			String[] spnew = {"TLSv1.2"};
//			sslserversocket.setEnabledProtocols(spnew);
			
			
/*			
			System.out.println("Enabled ciphers: " + Arrays.toString(sslserversocket.getEnabledCipherSuites()));
*/
			
			System.out.println("startTLSclient: Start TLS handshake");
			sslsocket.startHandshake();
			
			InputStream inputstream = sslsocket.getInputStream();
			OutputStream outputstream = sslsocket.getOutputStream();
	
			InputStreamReader inputstreamreader = new InputStreamReader(
					inputstream);
			BufferedReader bufferedreader = new BufferedReader(
					inputstreamreader);

			OutputStreamWriter outputstreamwriter = new OutputStreamWriter(
					outputstream);
			BufferedWriter bufferedwriter = new BufferedWriter(
					outputstreamwriter);

			System.out.println("startTLSclient: send test string");
			bufferedwriter.write("test string");
			bufferedwriter.flush();

			String sslStr;
			while (true) {
				sslStr = bufferedreader.readLine();
				System.out.println(sslStr);
				System.out.flush();
				if(sslStr.startsWith("close")){
					System.out.println("startTLSclient: received close string");
					break;
				}
			}

			
			System.out.println("startTLSclient: close in out streams");
			bufferedwriter.close();
			outputstreamwriter.close();
			outputstream.close();
			bufferedreader.close();
			inputstreamreader.close();
			inputstream.close();

		}
		catch (Exception err) {
			System.out.println("startTLSclient: " + err);
			err.printStackTrace();
		}
		finally
		{
			if(sslsocket != null ) { 
				System.out.println("startTLSclient: Connection to " + sslsocket.getRemoteSocketAddress() + " closed");
				sslsocket.close();
			}
		}
	}

}