import java.io.*;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.GeneralSecurityException;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.NoSuchAlgorithmException;
 
import javax.net.ssl.KeyManager;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;


public class TLSclientBrowser {
	
	private static SSLContext sc = null;
	private static SSLSocketFactory sslsocketfactory = null;
	
	private static KeyManager[] getKeyManagerArray(String keystore,
			String keystorePasswd) {
		KeyManager[] ret = null;
		String keyFile = "keystore";
 
		if (null != keystore) {
			keyFile = keystore;
		} else
			keyFile = "keystore.jks";
		
		File t = new File(keyFile);
		if (!t.exists()) {
			throw new RuntimeException("Could not find key manager file");
		}
		
		if (null == keystorePasswd) {
			keystorePasswd = "123456";
		}
		try {
			System.out.println("Using keystore: " + keyFile);
			KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509", "SunJSSE");
			KeyStore ks = KeyStore.getInstance("JKS");
			// initialize KeyStore object using keystore name
			ks.load(new FileInputStream(keyFile), keystorePasswd.toCharArray());
			kmf.init(ks, keystorePasswd.toCharArray());
			ret = kmf.getKeyManagers();
		} catch (Exception e) {
			System.out.println("getKeyManagerArray Exception");
			e.printStackTrace();
		}
 
		return ret;
	}
	
	private static SSLContext getSSLContext() throws NoSuchAlgorithmException, KeyManagementException {
		SSLContext ret = SSLContext.getInstance("TLSv1.2");
//		SSLContext ret = SSLContext.getInstance("TLSv1");
		//		KeyManager[] km = getKeyManagerArray(null, null);
		KeyManager[] km = getKeyManagerArray("keystore", "secret");
//		KeyManager[] km = getKeyManagerArray("keystoreclient.jks", "clientpwd");
		ret.init(km, null, null);
		return ret;
	}
	
	public static void main(String[] arstring) throws GeneralSecurityException,  IOException {

		ServerSocket serversocket = null;
		
		System.setProperty("javax.net.debug", "all");
		java.lang.System.setProperty("sun.security.ssl.allowUnsafeRenegotiation", "true");

		try {
//			sc = getSSLContext();
			serversocket = new ServerSocket();
			serversocket.setReuseAddress(true);
			serversocket.bind(new InetSocketAddress("127.0.0.2",5555));
			System.out.println("Create agent socket 127.0.0.2:5555");

//			sslsocketfactory = (SSLSocketFactory) SSLSocketFactory.getDefault();
//			sslsocketfactory = (SSLSocketFactory) sc.getSocketFactory();

			while(true)
			{
				waitclient(serversocket);
			}
		}
		catch (Exception e) {
			System.out.println(e.getMessage());
        }
	}
		
	private static void waitclient(ServerSocket server) throws IOException {
		
		System.out.println("agent accept");
		Socket sock = server.accept();
		System.out.println("waitclient: Connection from " + sock.getRemoteSocketAddress());
		Process p = null;
		
		InputStream inputstream = sock.getInputStream();
		OutputStream outputstream = sock.getOutputStream();

		InputStreamReader inputstreamreader = new InputStreamReader(
				inputstream);
		BufferedReader bufferedreader = new BufferedReader(
				inputstreamreader);

		OutputStreamWriter outputstreamwriter = new OutputStreamWriter(
				outputstream);
		BufferedWriter bufferedwriter = new BufferedWriter(
				outputstreamwriter);
		System.out.println("Create input-output channels");

		try {
			String str = null;
			String dest_ip = null;
			String dest_port = null;
			boolean waitInputString = true;
			
			while (waitInputString) {
				str = bufferedreader.readLine();
				System.out.println(str);
				System.out.flush();
				if(p != null) {
					System.out.println("waitclient: previous proccess running, stop it");
					p.destroyForcibly();
					p = null;
				}
				if(str.startsWith("close")){
					System.out.println("waitclient: received close string");
					break;
				}
				if(!str.startsWith("start TLS to ")){
					System.out.println("waitclient: wait string: 'start TLS to <ip_address:port>'");
					System.out.println("waitclient: wait string: 'start TLS to <port>'");
					bufferedwriter.write("waitclient: wait string: 'start TLS to addr <ip_address:port>'");
					bufferedwriter.write("waitclient: wait string: 'start TLS to port <port>'");
					break;
				}
				String addr_dest = str.substring(13);
				int ind = addr_dest.indexOf(':');
				if(ind < 0) {
					dest_port = addr_dest;
				}
				else if(addr_dest.length() < ind + 2) {
					System.out.println("waitclient: wait string: 'start TLS to <ip_address:port>'");
					System.out.println("waitclient: wait string: 'start TLS to <port>'");
					bufferedwriter.write("waitclient: wait string: 'start TLS to addr <ip_address:port>'");
					bufferedwriter.write("waitclient: wait string: 'start TLS to port <port>'");
					break;
				}
				else {
					dest_ip = addr_dest.substring(0, ind);
					dest_port = addr_dest.substring(ind+1);
				}
				
				if((dest_ip == null) || (dest_ip.length() == 0)) {
					System.out.println("startTLSclient: server ip_address not defined, get remote socket address");
					dest_ip = sock.getInetAddress().getHostAddress();
				}
				startTLSclient(dest_ip,dest_port,p);
			}
			System.out.println("waitclient: Connection from " + sock.getRemoteSocketAddress() + " closed");
			outputstreamwriter.close();
			bufferedwriter.close();
			sock.close();
		} catch (Exception exception) {
			exception.printStackTrace();
		}
		
	}
	
	
	private static void startTLSclient(String serv_ip, String serv_port, Process p) throws IOException {
		
		if(serv_ip == null || serv_port == null) {
			System.out.println("startTLSclient: server ip_address or port not defined");
			return;
		}

		Runtime r = Runtime.getRuntime();
		
		try {
//			p = r.exec("firefox -new-tab \"https://192.168.56.1:7777\"");
//			p = r.exec("y:\\FirefoxPortable8502_clean_\\App\\Firefox64\\firefox.exe \"https://127.0.0.1:12350\"");
//			p = r.exec("y:\\ChromiumPortable103\\app\\chrome.exe \"https://127.0.0.1:12350\"");
//			p = r.exec("C:\\Users\\alexn.INTRA\\AppData\\Local\\Yandex\\YandexBrowser\\Application\\browser.exe \"https://127.0.0.1:12350\"");
			System.out.println("startTLSclient: run browser");
			p = r.exec("C:\\Users\\alexn.INTRA\\AppData\\Local\\Mail.Ru\\Atom\\Application\\atom.exe -new-tab \"https://127.0.0.1:12350\"");
			
//			p.waitFor();
			System.out.println("startTLSclient: continue");
		}
		catch (Exception e) {
			System.out.println(e);
		}
		
		System.out.println("startTLSclient close");
	}

}