package ru.ispras.telecomtesk.tlsv13.HandshakeMessages;

import java.util.ArrayList;

import ru.ispras.telecomtesk.tlsv13.ConvertTypes;
import ru.ispras.telecomtesk.tlsv13.DebugInfo;
import ru.ispras.telecomtesk.tlsv13.HandshakeMessages.TLSExtensions.TLSExtension;
import ru.ispras.telecomtesk.tlsv13.HandshakeMessages.TLSExtensions.TLSExtensionsList;

public class CertificateRequest extends HandshakeMessage
{
	private byte[] context;    // <0..2^8-1>
	private ArrayList<TLSExtension> extensions; // <2..2^16-1> 
	
	public CertificateRequest() {
		this(null, null);
	}

	public CertificateRequest(byte[] context) {
		this(context, null);
	}

	public CertificateRequest(byte[] context, ArrayList<TLSExtension> extensions) {
		msg_type = 13;
		this.context = context != null ? context : new byte[0];
		this.extensions = extensions != null ? extensions : new ArrayList<TLSExtension>();
	}

	public byte[] getContext() {
		return context;
	}

	@Override
	public int getSize() {
		return (4 + 1 + context.length + 2 + TLSExtensionsList.getExtensionLength(extensions));
	}

	public ArrayList<TLSExtension> getExtensions() {
		return extensions;
	}

	public void addExtension(TLSExtension ext) {
		extensions.add(ext);
	}
	
	public void removeExtension(TLSExtension ext) {
		extensions.remove(ext);
	}
	
	public void clearExtensionList() {
		extensions.clear();
	}

	@Override
	public String toString() {
		String res;

		res = "HandshakeMessage: type CertificateRequest, length " + length;
		if(net_msg != null)
				res += ", net_msg: " + net_msg.length;

		return res;
	}

	@Override
	public void print() {
		DebugInfo.printMessage(1,"\nHandshake CertificateRequest Message:");
		DebugInfo.printDump(1, context, "context:");
		TLSExtensionsList.print(this.extensions);
	}

	@Override
	public int toByteArray() {
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		// write msg_type and full length
		byte[] ba_len = ConvertTypes.hton(length);
		ba[0] = msg_type;
		System.arraycopy(ba_len, 1, ba, 1, 3);
		int i = 4;
		// CertificateRequest context
		ba[i++] = (byte)context.length;
		System.arraycopy(context, 0, ba, i, context.length);
		i += context.length;
		//write extensions
		TLSExtensionsList.toByteArray(ba, i, extensions);
		
		net_msg = ba;
		return 0;
	}
	
	public static HandshakeMessage parseByteArray(byte[] msg) {
    	int pos = 4, curlen;

    	// parse CertificateRequestContext
    	if(msg.length <= pos) {
    		DebugInfo.printMessage(4,"===Error! CertificateRequest::parseByteArray: CertificateRequestContext length failed");
    		return null;
		}
    	curlen = msg[pos++];
    	if(msg.length - pos < curlen) {
    		DebugInfo.printMessage(4,"===Error! CertificateRequest::parseByteArray: CertificateRequestContext length wrong");
    		return null;
		}
    	byte[] context = new byte[curlen];
		System.arraycopy(msg, pos, context, 0, curlen);
		pos += curlen;
    	
    	// parse Extensions
    	ArrayList<TLSExtension> extensions = TLSExtensionsList.parseTLSExtensionsList(msg, pos);
    	
		return new CertificateRequest(context, extensions);
	}
	    	
}
