package ru.ispras.telecomtesk.tlsv13.HandshakeMessages;

import java.lang.System;
import java.util.ArrayList;

import ru.ispras.telecomtesk.tlsv13.ConvertTypes;
import ru.ispras.telecomtesk.tlsv13.DebugInfo;
import ru.ispras.telecomtesk.tlsv13.HandshakeMessages.TLSExtensions.*;

public class EncryptedExtensions extends HandshakeMessage
{
	private ArrayList<TLSExtension> extensions; // 0..2^16-1 

	public EncryptedExtensions() {
		this(null); 
	}

	public EncryptedExtensions(ArrayList<TLSExtension> extensions) {
		msg_type = 8;
		this.extensions = extensions != null ? extensions : new ArrayList<TLSExtension>(); 
	}

	@Override
	public int getSize() {
		return (4 + 2 + TLSExtensionsList.getExtensionLength(extensions));
	}

	public ArrayList<TLSExtension> getExtensions() {
		return extensions;
	}

	public void addExtension(TLSExtension ext) {
		extensions.add(ext);
	}
	
	public void removeExtension(TLSExtension ext) {
		extensions.remove(ext);
	}
	
	public void clearExtensionList() {
		extensions.clear();
	}

	@Override
	public int toByteArray() {
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		// write msg_type and full length
		byte[] ba_len = ConvertTypes.hton(length);
		ba[0] = msg_type;
		System.arraycopy(ba_len, 1, ba, 1, 3);
		int i = 4;
		//write extensions
		TLSExtensionsList.toByteArray(ba, i, extensions);
		
		net_msg = ba;
		return 0;
	}

	@Override
	public String toString() {
		String res;

		res = "HandshakeMessage: type EncryptedExtensions, length " + length;
		if(net_msg != null)
				res += ", net_msg: " + net_msg.length;

		return res;
	}

	@Override
	public void print() {
		DebugInfo.printMessage(1,"\nHandshake EncryptedExtensions Message:");
		TLSExtensionsList.print(this.extensions);
	}

	public static HandshakeMessage parseByteArray(byte[] msg) {
    	ArrayList<TLSExtension> extensions = TLSExtensionsList.parseTLSExtensionsList(msg, 4);

		return new EncryptedExtensions(extensions);
	}

}
