package ru.ispras.telecomtesk.tlsv13.HandshakeMessages;

import java.util.Random;
import ru.ispras.telecomtesk.tlsv13.ConvertTypes;
import ru.ispras.telecomtesk.tlsv13.DebugInfo;
import ru.ispras.telecomtesk.tlsv13.TLSState.MessageDirection;
import ru.ispras.telecomtesk.tlsv13.TLSState.TLSConnection;
import ru.ispras.telecomtesk.tlsv13.TLSState.TLSKeyDerivation;
import ru.ispras.telecomtesk.tlsv13.TLSState.TLSSecurityParameters;
import ru.ispras.telecomtesk.tlsv13.TLSAlgorithms.MACAlgorithm;
import java.util.ArrayList;

public class Finished extends HandshakeMessage
{
	String[] finished_label = {"client finished","server finished"}; 
	public byte[] verify_data;

	public Finished() {
		msg_type = 20;
		verify_data = new byte[0];
	}
	
	public Finished(byte[] data) {
		msg_type = 20;
		setVerifyData(data);
	}
	
	public Finished(TLSConnection con, MessageDirection msgDir) {
		msg_type = 20;
		try {
			setVerifyData(con, msgDir); 
		}
		catch (Exception err) {
			DebugInfo.printMessage(5,"===Warning! Finished: new message setVerifyData failed. Create random VerifyData");
			this.setRandomVerifyData(con);
		}
		
	}
	
	public byte[] calcVerifyData(TLSConnection con, byte[] handshake_traffic_secret) throws Exception {
		byte[] macKey;
		MACAlgorithm malg;
		Object curmsg;
		
		DebugInfo.printMessage(5,"Finished::calcVerifyData start");
		
		DebugInfo.printDump(5, handshake_traffic_secret, "handshake_traffic_secret: ");
		ArrayList<HandshakeMessage> handshakeContext = new ArrayList<HandshakeMessage>();
		ArrayList msglist = con.getHandshakeParams().sessionMsgList;

		for(int i=0;i<msglist.size();i++) {
			curmsg = msglist.get(i);
			if(curmsg instanceof HandshakeMessage) {
				if(curmsg instanceof HandshakeUndefinedType) continue;  // TEST message type, not include in Hash calculating
				handshakeContext.add((HandshakeMessage)curmsg);
			}
		}

		malg = con.getSecurityParams().mac_alg;
		byte[] hashHandshakeContext = TLSKeyDerivation.TLS_TranscriptHash(malg, handshakeContext);
		macKey = TLSKeyDerivation.TLS_HKDF_ExpandLabel(malg, handshake_traffic_secret, "finished", null);
		
		return TLSKeyDerivation.mac(malg,macKey,hashHandshakeContext);
	}

	public void setVerifyData(byte[] vdata) {
		verify_data = vdata != null ? vdata : new byte[0];
	}
	
	public void setVerifyData(TLSConnection con, byte[] handshake_traffic_secret) throws Exception {
		verify_data = calcVerifyData(con,handshake_traffic_secret);
	}
	
	public void setVerifyData(TLSConnection con, MessageDirection msgDir) throws Exception {
		verify_data = calcVerifyData(con, con.getHandshakeTrafficSecret(msgDir));
	}
	
	public void setRandomVerifyData(TLSConnection con) {
		setRandomVerifyData(con.getSecurityParams().mac_alg.getHashLength());
	}
	
	public void setRandomVerifyData(int len) {
		verify_data = new byte[len];
		Random rndObj = new Random(System.currentTimeMillis());
		rndObj.nextBytes(verify_data);
	}
	
	public int getSize() {
		return 4 + verify_data.length;
	}

	@Override
	public void print() {
		DebugInfo.printMessage(1,this.toString());
		DebugInfo.printDump(1,this.verify_data,"verify_data:");
	}

	public int toByteArray() {
		int res = 0;
		if(isLengthSet == false) setLength();
		byte[] ba = new byte[getSize()];
		byte[] ba_len = ConvertTypes.hton(length);
		ba[0] = msg_type;
		System.arraycopy(ba_len, 1, ba, 1, 3);
		System.arraycopy(verify_data, 0, ba, 4, verify_data.length);
		net_msg = ba;
		return res;
	}
	
    public static HandshakeMessage parseByteArray(byte[] msg, boolean IsResponseFromServer) {
    	HandshakeMessage hm = null;

    	if(msg.length >= 4) {
        	byte[] vr = new byte[msg.length - 4];
        	System.arraycopy(msg, 4, vr, 0, vr.length);
    		hm = new Finished(vr);
		}
    	else {
    		DebugInfo.printMessage(4,"===Error! Finished::parseByteArray: message length failed: " + msg.length);
    	}

    	return hm;
    }

}