package ru.ispras.telecomtesk.tlsv13.HandshakeMessages;

import java.util.ArrayList;

import ru.ispras.telecomtesk.tlsv13.ConvertTypes;
import ru.ispras.telecomtesk.tlsv13.DebugInfo;
import ru.ispras.telecomtesk.tlsv13.HandshakeMessages.TLSExtensions.TLSExtensionsList;
import ru.ispras.telecomtesk.tlsv13.Mutation.MutateField;
import ru.ispras.telecomtesk.tlsv13.TLSAlgorithms.*;
import ru.ispras.telecomtesk.tlsv13.TLSState.ConnectionRole;
import ru.ispras.telecomtesk.tlsv13.TLSState.TLSConnection;

public class HandshakeMessage
{
	@MutateField
	public byte msg_type;
	@MutateField
	public int length;
	protected boolean isLengthSet = false;
	public byte[] net_msg = null; // byte array representation of message
	
	public void setMessageType(byte type) {
		msg_type = type;
	}
	public int getSize() {
		return 4;
	}
	public void setLength() {
		length = getSize() > 4 ? getSize() - 4 : 0;
		isLengthSet = true;
	}
	public void setLength(int len) {
		length = len;
		isLengthSet = true;
	}
	public int getLength() {
		return length;
	}
	public String toString() {
		String res = "HandshakeMessage: type " + HandshakeMessageType.toString(msg_type) + ", length " + length;
		if(net_msg != null)
			res += ", net_msg: " + net_msg.length;
		return res;
	}
	public int toByteArray() {
		if(isLengthSet == false) setLength();
		net_msg = ConvertTypes.hton(length);
		net_msg[0] = msg_type;
		return 0;
	}
    
	public void print() {}
	
    public static int parseHandshakeMessagesList(byte[] msgs, ArrayList msglst, TLSConnection con) {
    	int i,offset,hdrlen;
    	byte[] msgByteArray;
    	HandshakeMessage curHM;
    	int err = 0;
    	
    	if((msgs != null) && (msglst != null)) {
    		for(i=0,offset = msgs.length; i < msgs.length; ) {
    			if(offset < 4) {
    				err = -1;
					DebugInfo.printMessage(3,"===Error! HandshakeMessage::parseHandshakeMessagesList: Handshake Message header length is wrong");
    				break;
    			}
    			// get length of current Handshake Message
    			hdrlen = 4 + ConvertTypes.ntoh(msgs, i+1, 3);
    			if(offset < hdrlen) {
    				err = -2;
    				DebugInfo.printMessage(3,"===Error! HandshakeMessage::parseHandshakeMessagesList: Handshake Message length is wrong");
    				break;
    			}
    			msgByteArray = new byte[hdrlen];
				System.arraycopy(msgs, i, msgByteArray, 0, hdrlen);
				curHM = parseHandshakeMessage(msgByteArray, con);
				if(curHM != null)
					msglst.add(curHM);
				else
					break;
				
				i += hdrlen;
				offset -= hdrlen;
    		}
    	}
    	return err;
    }

    
    public static HandshakeMessage parseHandshakeMessage(byte[] msg, TLSConnection con) {
    	HandshakeMessage hm = null;
    	int len = 0;

		if(msg.length < 4) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHandshakeMessage: header length failed");
    		return hm;
		}
		// get length of current Handshake Message
		len = ConvertTypes.ntoh(msg, 1, 3);
		if(len != msg.length - 4) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHandshakeMessage: message length failed");
    		return hm;
		}
		
    	switch(msg[0]) {
    	case 0: hm = HelloRequest.parseByteArray(msg); break;
    	case 1: 
    	case 2: 
    			hm = Hello.parseByteArray(msg);
    			if((hm != null) && (msg[0] == 2) && !(hm instanceof HelloRetryRequest)) {
    				con.getSecurityParams().setAlgorithmParams(TLSCipherSuite.valueOf(((Hello)hm).cipher_suites[0]));
    			}
    			break;
    	case 4: hm = NewSessionTicket.parseByteArray(msg); break;
    	case 5: hm = EndOfEarlyData.parseByteArray(msg); break;
    	case 8: hm = EncryptedExtensions.parseByteArray(msg); break;
    	case 11: hm = HMCertificate.parseByteArray(msg); break;
    	case 12: hm = ServerKeyExchange.parseByteArray(msg, con.getSecurityParams().ke_alg); break;
    	case 13: hm = CertificateRequest.parseByteArray(msg); break;
    	case 14: hm = ServerHelloDone.parseByteArray(msg); break;
    	case 15: hm = CertificateVerify.parseByteArray(msg); break;
    	case 16: hm = ClientKeyExchange.parseByteArray(msg, con.getSecurityParams().ke_alg); break;
    	case 20: hm = Finished.parseByteArray(msg, (con.localRole == ConnectionRole.CLIENT)); break;  
    	case 24: hm = KeyUpdate.parseByteArray(msg); break;  
    	default:
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHandshakeMessage: unknown message type " + msg[0]);
    	}

    	if(hm != null) {
			hm.print();
    		hm.setLength(len);
    		hm.net_msg = msg;
    		
    		len = hm.getSize();
       		if(msg.length > len) {
        		DebugInfo.printDumpWithLength(1, msg, len, "===Error! HandshakeMessage::parseByteArray: some bytes remain");   			
       		}

    	}
    	return hm;
    }

}

