package ru.ispras.telecomtesk.tlsv13.HandshakeMessages.TLSExtensions;

import java.util.ArrayList;

import ru.ispras.telecomtesk.tlsv13.ConvertTypes;
import ru.ispras.telecomtesk.tlsv13.DebugInfo;
import ru.ispras.telecomtesk.tlsv13.TLSAlgorithms.NamedGroup;

public class KeyShareClientHello extends TLSExtension
{
	public ArrayList<KeyShareEntry> data; // <0..2^16-1> bytes 

	public KeyShareClientHello() {
		this.type = 51;
		this.data = new ArrayList<KeyShareEntry>();
	}

	public KeyShareClientHello(ArrayList<KeyShareEntry> dt) {
		this.type = 51;
		if(dt != null)
			this.data = dt;
		else
			this.data = new ArrayList<KeyShareEntry>();
	}

	public KeyShareClientHello(KeyShareEntry dt) {
		this();
		this.data.add(dt);
	}

	public KeyShareClientHello(int length , ArrayList<KeyShareEntry> dt) {
		this(dt);
		this.setLength(length);
	}

	@Override
	public int getLengthFieldSize() {
		return 2;
	}

	@Override
	public int getDataSize() {
		int res = 0;  
		if(data != null) {
			for (int j=0; j < data.size(); j++) {
				res += (data.get(j)).getSize();
			}
		}
		return res;
	}

	@Override
	public void toByteArrayData(byte[] msg, int position) {
		if(data != null) {
			for (int j=0; j < data.size(); j++) {
                int curPos = (data.get(j)).toByteArray(msg,position);
				if(curPos >= 0) 
					position = curPos;
				else 
					DebugInfo.printMessage(1,"!!! Error: KeyShareClientHello toByteArray return null, [" + j + "]");
			}
		}
	}

	@Override
	public void printData() {
		DebugInfo.printMessage(1,"  KeyShareClientHello");
		if(data != null) {
			for(int j=0; j<data.size(); j++)
				(data.get(j)).print();
		}
	}

	public static TLSExtension parseByteArray(int lengthExt, byte[] msg, int pos) {
		int len;
		int lengthFieldSize = 2;
    	if(lengthExt < lengthFieldSize) {
    		DebugInfo.printMessage(4,"===Error! KeyShareClientHello::parseByteArray: Extension length failed");
    		return null;
    	}
		len = ConvertTypes.ntohs(msg,pos);
		pos += lengthFieldSize;

    	if(lengthExt != lengthFieldSize + len) {
    		DebugInfo.printMessage(4,"===Error! KeyShareClientHello::parseByteArray: Extension length wrong");
    		return null;
    	}
    	
    	ArrayList<KeyShareEntry> extdata = new ArrayList<KeyShareEntry>();
    	int keylen;

		while(len > 0) {
    		KeyShareEntry curKeyEntry = KeyShareEntry.parseByteArray(msg,pos);
			if(curKeyEntry == null) 
				return null;
    		extdata.add(curKeyEntry);
    		keylen = curKeyEntry.getSize();
    		pos += keylen;
    		len -= keylen;
		}

//		params = set cookie;
		
		return new KeyShareClientHello(lengthExt, extdata);
	}
}
