package ru.ispras.telecomtesk.tlsv13.HandshakeMessages.TLSExtensions;

import ru.ispras.telecomtesk.tlsv13.ConvertTypes;
import ru.ispras.telecomtesk.tlsv13.DebugInfo;

public class PskIdentity
{
	public int obfuscatedTicketAge; // uint32
	public byte[] identity; // <1..2^16-1> bytes

	public PskIdentity(int obfuscatedTicketAge, byte[] identity) {
		this.obfuscatedTicketAge = obfuscatedTicketAge; 
		this.identity = identity;
	}

	public int getSize() {
		if(identity != null)
			return (6 + identity.length);
		else
			return 6;
	}

	public byte[] toByteArray() {
		int i = 0;
		byte[] res = new byte[getSize()];
		int len = res.length - 6;
		res[i++] = (byte)(len >>> 8);
		res[i++] = (byte)len;
		if(identity != null) {
			System.arraycopy(identity, 0, res, i, identity.length);
			i+=identity.length;
		}
		byte[] ta = ConvertTypes.hton(obfuscatedTicketAge);
		System.arraycopy(ta, 0, res, i, ta.length);
		return res;
	}

	public int toByteArray(byte[] msg, int position) {
		int len = this.getSize();
		if((msg == null) || (position < 0) || (msg.length - position < len))
			return -1;

		len -= 6;
		msg[position++] = (byte)(len >>> 8);
		msg[position++] = (byte)len;
		if(identity != null)
			System.arraycopy(identity, 0, msg, position, identity.length);
		position += len;
		byte[] ta = ConvertTypes.hton(obfuscatedTicketAge);
		System.arraycopy(ta, 0, msg, position, ta.length);
		position += ta.length;

		return position;
	}

	public void print() {
		DebugInfo.printMessage(1,"  PskIdentity: obfuscatedTicketAge " + obfuscatedTicketAge);
		DebugInfo.printDump(1,identity,"  identity:");
	}

	public static PskIdentity parseByteArray(byte[] msg, int pos) {
		int len;
		int lengthExt = msg.length - pos;
    	if(lengthExt < 6) {
    		DebugInfo.printMessage(4,"===Error! PskIdentity::parseByteArray: length failed");
    		return null;
    	}

    	len = ConvertTypes.ntohs(msg,pos);
		pos += 2;

    	if(lengthExt < 6 + len) {
    		DebugInfo.printMessage(4,"===Error! PskIdentity::parseByteArray: Extension length wrong");
    		return null;
    	}
    	
    	byte[] identity = new byte[len];
		System.arraycopy(msg, pos, identity, 0, len);
    	pos += len;
		
    	int obfuscatedTicketAge = ConvertTypes.ntoh(msg, pos);
		
		return new PskIdentity(obfuscatedTicketAge,identity);
	}
}
