package ru.ispras.telecomtesk.tlsv13.HandshakeMessages.TLSExtensions;

import ru.ispras.telecomtesk.tlsv13.ConvertTypes;
import ru.ispras.telecomtesk.tlsv13.DebugInfo;
import ru.ispras.telecomtesk.tlsv13.TLSAlgorithms.HandshakeMessageType;

// RFC 8449:  Record Size Limit Extension for TLS
public class RecordSizeLimitExt extends TLSExtension
{
	int RecordSizeLimit;
	
        // data <1..2^16-1>
	public RecordSizeLimitExt() {
		type = 28; 
		this.data = null;
		this.RecordSizeLimit = 16384;
	}

	public RecordSizeLimitExt(int RecordSizeLimit) {
		type = 28; 
		this.RecordSizeLimit = RecordSizeLimit;
	}

	public RecordSizeLimitExt(int length, int RecordSizeLimit) {
		this(RecordSizeLimit); 
		this.setLength(length);
	}

	@Override
	public int getDataSize() {
		return 2;
	}

	@Override
	public void toByteArrayData(byte[] msg, int position) {
		msg[position++] = (byte)(RecordSizeLimit >>> 8);
		msg[position++] = (byte)(RecordSizeLimit);
	}

	@Override
	public void printData() {
		String res = "  value: " + RecordSizeLimit;
		DebugInfo.printMessage(1,res);
	}

	public static TLSExtension parseByteArray(int lengthExt, byte[] msg, int pos) {
    	if(lengthExt != 2) {
    		DebugInfo.printMessage(4,"===Error! RecordSizeLimitExt::parseByteArray: Extension length " + lengthExt + ", must be 2 bytes");
    		return null;
    	}
    	
		int rsl = ConvertTypes.ntohs(msg,pos);

		return new RecordSizeLimitExt(lengthExt, rsl);
	}
}
