package ru.ispras.telecomtesk.tlsv13.HandshakeMessages.TLSExtensions;

import ru.ispras.telecomtesk.tlsv13.ConvertTypes;
import ru.ispras.telecomtesk.tlsv13.DebugInfo;

public class ServerName
{
	public byte nameType; // enum { host_name(0), (255) } NameType;
	public byte[] name; // <1..2^16-1> bytes

	public ServerName(byte[] name) {
		this((byte)0, name); 
	}

	public ServerName(String name) {
		this((byte)0, name.getBytes()); 
	}

	public ServerName(byte nameType, byte[] name) {
		this.nameType = nameType; 
		this.name = name;
	}

	public int getSize() {
		if(name != null)
			return (3 + name.length);
		else
			return 3;
	}

	public byte[] toByteArray() {
		int i = 0;
		int len = getSize();
		byte[] res = new byte[len];
		len -= 3;
		res[i++] = nameType;
		res[i++] = (byte)(len >>> 8);
		res[i++] = (byte)len;
		if(name != null) {
			System.arraycopy(name, 0, res, i, name.length);
		}
		return res;
	}

	public int toByteArray(byte[] msg, int position) {
		int len = this.getSize();
		if((msg == null) || (position < 0) || (msg.length - position < len))
			return -1;

		len -= 3;
		msg[position++] = nameType;
		msg[position++] = (byte)(len >>> 8);
		msg[position++] = (byte)len;
		if(name != null)
			System.arraycopy(name, 0, msg, position, name.length);
		position += len;

		return position;
	}

	public void print() {
		DebugInfo.printMessage(1,"  ServerName: nameType " + nameType);
		DebugInfo.printDump(1,name,"  name:");
	}

	public static ServerName parseByteArray(byte[] msg, int pos) {
		int len;
		int lengthExt = msg.length - pos;
    	if(lengthExt < 3) {
    		DebugInfo.printMessage(4,"===Error! ServerName::parseByteArray: length failed");
    		return null;
    	}
    	
    	byte nameType  = msg[pos++];

    	len = ConvertTypes.ntohs(msg,pos);
		pos += 2;

    	if(lengthExt < 3 + len) {
    		DebugInfo.printMessage(4,"===Error! ServerName::parseByteArray: Extension length wrong");
    		return null;
    	}
    	
    	byte[] name = new byte[len];
		System.arraycopy(msg, pos, name, 0, len);
    	pos += len;
		
		return new ServerName(nameType,name);
	}
}
