package ru.ispras.telecomtesk.tlsv13.HandshakeMessages.TLSExtensions;

public enum TLSExtensionType {
        SERVER_NAME(0),                             /* RFC 6066 */
        MAX_FRAGMENT_LENGTH(1),                     /* RFC 6066 */
        STATUS_REQUEST(5),                          /* RFC 6066 */
        SUPPORTED_GROUPS(10),                       /* RFC 8422, 7919 */
        SIGNATURE_ALGORITHMS(13),                   /* RFC 8446 */
        USE_SRTP(14),                               /* RFC 5764 */
        HEARTBEAT(15),                              /* RFC 6520 */
        APPLICATION_LAYER_PROTOCOL_NEGOTIATION(16), /* RFC 7301 */
        SIGNED_CERTIFICATE_TIMESTAMP(18),           /* RFC 6962 */
        CLIENT_CERTIFICATE_TYPE(19),                /* RFC 7250 */
        SERVER_CERTIFICATE_TYPE(20),                /* RFC 7250 */
        PADDING(21),                                /* RFC 7685 */
        RECORD_SIZE_LIMIT(28),                      /* RFC 7685 */
        PRE_SHARED_KEY(41),                         /* RFC 8446 */
        EARLY_DATA(42),                             /* RFC 8446 */
        SUPPORTED_VERSIONS(43),                     /* RFC 8446 */
        COOKIE(44),                                 /* RFC 8446 */
        PSK_KEY_EXCHANGE_MODES(45),                 /* RFC 8446 */
        CERTIFICATE_AUTHORITIES(47),                /* RFC 8446 */
        OID_FILTERS(48),                            /* RFC 8446 */
        POST_HANDSHAKE_AUTH(49),                    /* RFC 8446 */
        SIGNATURE_ALGORITHMS_CERT(50),              /* RFC 8446 */
        KEY_SHARE(51),                              /* RFC 8446 */
	UNKNOWN(65535);

	public short type;
	
	private TLSExtensionType(int type) {
		this.type = (short)type;
	}

	public short getvalue() {
		return type;
	}

	public static TLSExtensionType valueOf(int type) {
		switch(type){
		case  0: return SERVER_NAME;
		case  1: return MAX_FRAGMENT_LENGTH;
		case  5: return STATUS_REQUEST;
		case 10: return SUPPORTED_GROUPS;
		case 13: return SIGNATURE_ALGORITHMS;
		case 14: return USE_SRTP;
		case 15: return HEARTBEAT;
		case 16: return APPLICATION_LAYER_PROTOCOL_NEGOTIATION;
		case 18: return SIGNED_CERTIFICATE_TIMESTAMP;
		case 19: return CLIENT_CERTIFICATE_TYPE;
		case 20: return SERVER_CERTIFICATE_TYPE;
		case 21: return PADDING;
		case 28: return RECORD_SIZE_LIMIT;
		case 41: return PRE_SHARED_KEY;
		case 42: return EARLY_DATA;
		case 43: return SUPPORTED_VERSIONS;
		case 44: return COOKIE;
		case 45: return PSK_KEY_EXCHANGE_MODES;
		case 47: return CERTIFICATE_AUTHORITIES;
		case 48: return OID_FILTERS;
		case 49: return POST_HANDSHAKE_AUTH;
		case 50: return SIGNATURE_ALGORITHMS_CERT;
		case 51: return KEY_SHARE;
		default:
			System.out.println("===Warning! TLSExtensionType::valueOf: unknown type " + type);
			return UNKNOWN;
		}
	}

	public static String toString(int tp) {
		TLSExtensionType tmpval = TLSExtensionType.valueOf(tp);
		String value = "" + tmpval;
		String tmpstr = String.format("%04x", tp);
		if(tmpstr.length() > 4)
			tmpstr = tmpstr.substring(tmpstr.length()-4);
		value += "=0x" + tmpstr;
		return value;
	}
}
