package ru.ispras.telecomtesk.tlsv13.HandshakeMessages.TLSExtensions;

import java.util.ArrayList;

import ru.ispras.telecomtesk.tlsv13.ConvertTypes;
import ru.ispras.telecomtesk.tlsv13.DebugInfo;

public class TLSExtensionsList
{
	public static TLSExtension getExtension(ArrayList<TLSExtension> extensions, int index) {
		TLSExtension ext = null;
		try {
			ext = extensions.get(index);
		}
		catch (Exception ignored) {}
		
		return ext;
	}
	
	public static TLSExtension getExtension(ArrayList<TLSExtension> extensions, short type) {
		TLSExtension ext = null;
		try {
			if(extensions != null)
				for(int i = 0; i<extensions.size(); i++) {
					if(extensions.get(i).type == type) {
						ext = extensions.get(i);
						break;
					}
				}
		}
		catch (Exception ignored) {}
		
		return ext;
	}
	
	public static int getExtensionLength(ArrayList<TLSExtension> extensions) {
		int ext_len = 0;
		for (TLSExtension ext : extensions) {
			ext_len += ext.getSize();
		}
		return ext_len;
	}
	
	public static int indexOfExtension(ArrayList<TLSExtension> extensions, short type) {
		int res = -1;
		
		if(extensions != null) {
			for(int i=0; i<extensions.size(); i++) {
				if(type == ((TLSExtension)extensions.get(i)).type) {
					res = i;
					break;
				}
			}
		}
		return res;
	}
	
	public static int lastIndexOfExtension(ArrayList<TLSExtension> extensions, short type) {
		int res = -1;
		
		if(extensions != null) {
			for(int i=extensions.size()-1; i>=0; i--) {
				if(type == ((TLSExtension)extensions.get(i)).type) {
					res = i;
					break;
				}
			}
		}
		return res;
	}
	
	public static boolean checkDuplicateExtensions(ArrayList<TLSExtension> extensions) {
		TLSExtension curext;
		boolean res = false;
		
		if(extensions != null) {
			for(int i=0; i<extensions.size(); i++) {
				curext = (TLSExtension)extensions.get(i);
				if(i != lastIndexOfExtension(extensions, curext.type)) {
					res = true;
					break;
				}
			}
		}
		return res;
	}

	//     struct {
	//         uint16 length; full length of 'data'
	//         Extension[] data;
	//     } ExtensionsList;
	//
    //     pos - start position in the 'msg'
    public static ArrayList<TLSExtension> parseTLSExtensionsList(byte[] msg, int pos) {
    	return parseTLSExtensionsList(msg,pos,false);
    }
    
    public static ArrayList<TLSExtension> parseTLSExtensionsList(byte[] msg, int pos, boolean isHelloRetryMessage) {
    	ArrayList<TLSExtension> extensions = new ArrayList<TLSExtension>();
    	int HelloMsgType = isHelloRetryMessage ? 3 : msg[0];

    	if((pos >= 0)&&(msg.length > pos)) {
    		int extlen = msg.length - pos;
    		if(extlen >= 2) {
        		extlen = ConvertTypes.ntohs(msg,pos);
        		pos += 2;
        		if(extlen <= msg.length - pos) {
                	TLSExtension curext;
                	int len;
                	
            		while(extlen > 0) {
            			curext = TLSExtension.parseByteArray(msg, pos, HelloMsgType);
            			if(curext == null) break;
                    	extensions.add(curext);
                    	
                		pos+=2;
                		len = ConvertTypes.ntohs(msg,pos);
            			pos += 2 + len;
                		extlen -= 4 + len;
            		}
        		} else {
            		DebugInfo.printMessage(4,"===Error! TLSExtensionsList::parseTLSExtensionsList: Extensions length wrong");
        		}
    		} else {
        		DebugInfo.printMessage(4,"===Error! TLSExtensionsList::parseTLSExtensionsList: Extensions length failed");    			
    		}
    		
		}
		
    	return extensions;
    }

	// write extensions to byte array, beginning from 'pos'
    // no checks for NULL pointers
    //
	//     struct {
	//         uint16 length; full length of 'data'
	//         Extension[] data;
	//     } ExtensionsList;
	//
	public static void toByteArray(byte[] data, int pos, ArrayList<TLSExtension> extensions) {
		int j = TLSExtensionsList.getExtensionLength(extensions);
		data[pos++] = (byte)(j >>> 8);
		data[pos++] = (byte)j;
		byte[] extRaw;
		for (TLSExtension ext : extensions) {
			extRaw = ext.toByteArray();
			System.arraycopy(extRaw, 0, data, pos, extRaw.length);
			pos += extRaw.length;
		}
	}

	public static void print(ArrayList<TLSExtension> extensions) {
		DebugInfo.printMessage(1,"\nTLSExtensionsList:extensions: " + extensions.size());
		for(TLSExtension ext : extensions) 
			ext.print();
	}

}
