package ru.ispras.telecomtesk.tlsv13.RecordProtocol;

import java.math.BigInteger;
import java.security.*;

import javax.crypto.spec.DHParameterSpec;

import ru.ispras.telecomtesk.tlsv13.DebugInfo;
import ru.ispras.telecomtesk.tlsv13.TLSAlgorithms.SignatureAlgorithm;

public class GeneratePairKeys {

    static public KeyPair generateKeys(SignatureAlgorithm keyAlgorithm, int keySizeBits) 
    {
    	KeyPair keyPair = null;

        try {
            KeyPairGenerator keyGen = KeyPairGenerator.getInstance(keyAlgorithm.name());
            keyGen.initialize(keySizeBits);
            keyPair = keyGen.genKeyPair();
        } catch (NoSuchAlgorithmException e) {
			DebugInfo.printMessage(1,"===Error! GeneratePairKeys::generateKeys No such algorithm: " + keyAlgorithm);
        }

        return keyPair;
    }

    static public KeyPair generateKeysDH(byte[] ps, byte[] gs, int ls) 
    {
    	KeyPair keyPair = null;

        try {
			BigInteger p = new BigInteger(ps);
			BigInteger g = new BigInteger(gs);
		    DHParameterSpec param = new DHParameterSpec(p,g,ls);
            KeyPairGenerator keyGen = KeyPairGenerator.getInstance("DiffieHellman");
            keyGen.initialize(param);
            keyPair = keyGen.genKeyPair();
        } catch (Exception e) {
			DebugInfo.printMessage(1,"===Error! GeneratePairKeys::generateKeysDH: " + e);
        }

        return keyPair;
    }

    static public KeyPair generateKeysDH(int ls) 
    {
        	KeyPair kp = null;

            try {
            	AlgorithmParameterGenerator algorithmparametergenerator=AlgorithmParameterGenerator.getInstance("DH");
            	algorithmparametergenerator.init(ls,new SecureRandom());
            	AlgorithmParameters algorithmparameters=algorithmparametergenerator.generateParameters();
            	DHParameterSpec dhparameterspec=algorithmparameters.getParameterSpec(DHParameterSpec.class);
	            KeyPairGenerator keyGen = KeyPairGenerator.getInstance("DiffieHellman");
	            keyGen.initialize(dhparameterspec);
	            kp = keyGen.genKeyPair();
            } catch (Exception e) {
    			DebugInfo.printMessage(1,"===Error! GeneratePairKeys::generateKeysDH: " + e);
            }

        return kp;
    }
}