package ru.ispras.telecomtesk.tlsv13.TLSAlgorithms;

public enum SessionCertificateType {
	RSA_SIGN(1),
	DSS_SIGN(2),
	RSA_FIXED_DH(3),
	DSS_FIXED_DH(4),
	RSA_EPHEMERAL_DH_RESERVED(5),
	DSS_EPHEMERAL_DH_RESERVED(6),
	FORTEZZA_DMS_RESERVED(20),
	UNKNOWN(255);
	
	public byte type;
	
	private SessionCertificateType(int type) {
		this.type = (byte)type;
	}
	
	public byte getValue() {
		return type;
	}
	public static SessionCertificateType valueOf(byte tp) {
		switch (tp) {
		case 1: return RSA_SIGN;
		case 2:	return DSS_SIGN;
		case 3:	return RSA_FIXED_DH;
		case 4:	return DSS_FIXED_DH;
		case 5:	return RSA_EPHEMERAL_DH_RESERVED;
		case 6:	return DSS_EPHEMERAL_DH_RESERVED;
		case 20: return FORTEZZA_DMS_RESERVED;
		default: return UNKNOWN;
		}
	}
	public static SessionCertificateType valueOf(SignatureAlgorithm alg) {
		if(alg != null) {
			switch (alg) {
			case RSA: return RSA_SIGN;
			case DSA: return DSS_SIGN;
			default: return UNKNOWN;
			}
		}
		return UNKNOWN;
	}
	public static SessionCertificateType valueOf(SignatureAndHashAlgorithm sah) {
		if(sah != null) {
			switch (SignatureAlgorithm.valueOf(sah.signature)) {
			case RSA: return RSA_SIGN;
			case DSA: return DSS_SIGN;
			default: return UNKNOWN;
			}
		}
		return UNKNOWN;
	}
	public static boolean isFixedDH(SessionCertificateType ct) {
		switch (ct) {
			case RSA_FIXED_DH:
			case DSS_FIXED_DH:
				return true;
			default: 
				return false;
		}
	}
}
