package ru.ispras.telecomtesk.tlsv13.TLSSocket;

import java.io.*;
import java.net.*;
import ru.ispras.telecomtesk.tlsv13.DebugInfo;

public class TLSServerSocket implements Runnable 
{
    ServerSocket skt = null;
    Socket remoteAgent_skt = null;
    String startTLSclientPrefix = "start TLS to ";
    String startTLSclientRequest;
    BufferedReader bufferedreaderAgent = null;
    BufferedWriter bufferedwriterAgent = null;
    
    String mainThreadName = "acceptThread";
    boolean canCreateNewSockets = false;
    Thread t;
    public volatile boolean runSocketFlag = true;
    
	public TLSSocketsList childSockets;
  
    TLSResponse socketResponse = null;
    
    protected String clientIP = "";
    protected int clientPort;
    protected String localIP = null;
    protected int localPort;
    public long responseTimeout = 10000; // in milliseconds
    public long responseStartTimeout = 10000; // in milliseconds
    
    
    public TLSServerSocket(String server, int remotePort, int localPort) throws UnknownHostException, IOException {
    	this.clientIP = server;
    	this.clientPort = remotePort;
    	this.localPort = localPort;
    	socketResponse = new TLSResponse();
    	childSockets = new TLSSocketsList();
		t = new Thread(this,mainThreadName);
		t.start();
    }

    public TLSServerSocket(String server, int remotePort, int localPort, int timeout) throws UnknownHostException, IOException {
    	this.clientIP = server;
    	this.clientPort = remotePort;
    	this.localPort = localPort;
    	responseTimeout = timeout;
    	socketResponse = new TLSResponse();
    	childSockets = new TLSSocketsList();
		t = new Thread(this,mainThreadName);
		t.start();
    }

    public TLSServerSocket(String server, int remotePort, String localIP, int localPort) throws UnknownHostException, IOException {
    	this.clientIP = server;
    	this.clientPort = remotePort;
    	this.localIP = localIP;
    	this.localPort = localPort;
    	socketResponse = new TLSResponse();
    	childSockets = new TLSSocketsList();
		t = new Thread(this,mainThreadName);
		t.start();
    }

    public TLSServerSocket(String server, int remotePort, String localIP, int localPort, int timeout) throws UnknownHostException, IOException {
    	this.clientIP = server;
    	this.clientPort = remotePort;
    	this.localIP = localIP;
    	this.localPort = localPort;
    	responseTimeout = timeout;
    	socketResponse = new TLSResponse();
    	childSockets = new TLSSocketsList();
		t = new Thread(this,mainThreadName);
		t.start();
    }

	public String getIP() {
		return clientIP;
	}

	public int getPort() {
		return clientPort;
	}

	public long getTimeout() {
		return responseTimeout;
	}

	public void setTimeout(long timeout) throws SocketException {
		responseTimeout = timeout;
		if(skt != null)
			skt.setSoTimeout((int)responseTimeout);
	}

	public void initialize() throws UnknownHostException, IOException {
		DebugInfo.printMessage(3,"Create local socket,"  + " address " + this.localIP  + " port " + this.localPort);
		try {
			skt = new ServerSocket();
			skt.setReuseAddress(true);
//			skt.bind(new InetSocketAddress("127.0.0.1", this.localPort));
//			skt.bind(new InetSocketAddress("192.168.56.1", this.localPort));
			if (localIP == null) {
				DebugInfo.printMessage(1,"Warning! Local address not specified");
				skt.bind(new InetSocketAddress(this.localPort));
			} else
				skt.bind(new InetSocketAddress(localIP, this.localPort));

		}
		catch (IOException err){
			DebugInfo.printMessage(1,"Error! Could not listen port " + this.localPort);
			throw err;
		}
		DebugInfo.printMessage(4,"Listening port " + this.localPort);

		//		skt.setSoTimeout((int)responseTimeout);
		
		DebugInfo.printMessage(3,"Connecting " + this.clientIP + " port " + this.clientPort);
		try {
			remoteAgent_skt = new Socket(this.clientIP, this.clientPort);
			DebugInfo.printMessage(3,"Connected to " + remoteAgent_skt.getInetAddress().getHostAddress() + ":" + remoteAgent_skt.getPort());
			InputStream inputAgentStream = remoteAgent_skt.getInputStream();
			OutputStream outputAgentStream = remoteAgent_skt.getOutputStream();

			// 'start TLS to <ip_address:port>'
			// 'start TLS to <port>'
//			startTLSclientRequest = startTLSclientPrefix + remoteAgent_skt.getLocalAddress().getHostAddress() + ":" + skt.getLocalPort() + "\n";
			startTLSclientRequest = startTLSclientPrefix + skt.getLocalPort() + "\n";
			DebugInfo.printMessage(4,"string for send to agent: " + startTLSclientRequest);
			InputStreamReader inputAgentStreamreader = new InputStreamReader(inputAgentStream);
			bufferedreaderAgent = new BufferedReader(inputAgentStreamreader);
			OutputStreamWriter outputAgentStreamwriter = new OutputStreamWriter(outputAgentStream);
			bufferedwriterAgent = new BufferedWriter(outputAgentStreamwriter);
			remoteAgent_skt.setSoTimeout((int)responseTimeout);
			DebugInfo.printMessage(4,"Create input-output channels");
		}
		catch (IOException err){
			DebugInfo.printMessage(1,"Warning! remoteAgent connection failed");
			DebugInfo.printMessage(1,"Warning! continue without remoteAgent");
		}
		
		canCreateNewSockets = true;
	}
	
	public void run() {
		DebugInfo.printMessage(3,"TLSServerSocket:: run thread");
		
		try {
			initialize();
		}
		catch (Exception err){
			DebugInfo.printMessage(1,"Error! TLSServerSocket: initialize failed: " + err);
			runSocketFlag = false;
		}

   		int lastsockind = 1;
		Socket newsocket;

		while(runSocketFlag) {
			try {
				newsocket = skt.accept();
				DebugInfo.printMessage(1,"New connection from " + newsocket.getRemoteSocketAddress());
			} catch (Exception err) {
				runSocketFlag = false;
				DebugInfo.printMessage(1,"Error! TLSServerSocket: accept failed: " + err);
				break;
			}
			
			try {
				if(canCreateNewSockets && childSockets.canAddNewSocket()) {
			        TLSServerChildSocket chldSkt = new TLSServerChildSocket(newsocket, "socket " + lastsockind);
			        lastsockind++;
			        childSockets.add(chldSkt);
		        }
				else {
					DebugInfo.printMessage(1,"create new socket not permitted, socket is closed");
					newsocket.close();
				}
			} catch (IOException err) {
				DebugInfo.printMessage(1, "TLSServerSocket::Run: " + err);
			}
		}
		DebugInfo.printMessage(3,"TLSServerSocket: stop thread");
	}
	
	public void connect() throws UnknownHostException, IOException {
		if (canCreateNewSockets == false){
			DebugInfo.printMessage(4,"TLSServerSocket::connect: socket not ready, waiting...");
			try {
				Thread.sleep(responseStartTimeout);
			} catch (InterruptedException ignored) {
			}
		}
		DebugInfo.printMessage(4,"TLSServerSocket::connect: Ask client to start TLS connection");
		DebugInfo.printMessage(4,startTLSclientRequest);
		if (remoteAgent_skt != null){
			try {
				bufferedwriterAgent.write(startTLSclientRequest);
				bufferedwriterAgent.flush();
			}
			catch (IOException err){
				DebugInfo.printMessage(1,"Error! TLSServerSocket: connection to agent failed: " + err);
				throw err;
			}
		}
	}
    
	public void disconnect() {
		childSockets.renew();
	}
	
	public void stop() {
		DebugInfo.printMessage(5,"TLSServerSocket: try stop all sockets");
		childSockets.closeAll();
		DebugInfo.printMessage(3,"TLSServerSocket: childSockets stopped");
		if (remoteAgent_skt != null){
			try {
				bufferedwriterAgent.write("close\n");
				bufferedwriterAgent.flush();
	    		bufferedreaderAgent.close();
	    		bufferedwriterAgent.close();
				remoteAgent_skt.close();
			}
			catch (IOException ignored) {
			}
		}
		if (skt != null){
			try {
				skt.close();
			}
			catch (IOException ignored) {
			}
		}
		DebugInfo.printMessage(3,"TLSServerSocket::stop: copmlete");
	}

	public TLSResponse getResponse() {
    	DebugInfo.printMessage(4,"TLSServerSocket::getResponse");
    	return childSockets.getResponse();
    }

    public TLSResponse getResponseNewConnection() {
    	DebugInfo.printMessage(4,"TLSServerSocket::getResponseNewConnection");
    	return childSockets.getResponseNewConnection();
    }

    public void renew() {
    	DebugInfo.printMessage(4,"TLSServerSocket::renew");
    	childSockets.renew();
    }

    public boolean existNewConnection() {
    	return childSockets.newsocks > 0;
    }

    public void sendMessage(byte[] message) throws IOException
    {
    	DebugInfo.printMessage(4,"TLSServerSocket::sendMessage");
   		childSockets.sendMessage(message);
    }

    public void finilize() throws IOException {
    	try {
    		runSocketFlag = false;
    		bufferedreaderAgent.close();
    		bufferedwriterAgent.close();
    		remoteAgent_skt.close();
    		childSockets.closeAll();
    		skt.close();
    	} catch (IOException ignored) {
    	}
    }
  
}