package ru.ispras.telecomtesk.tlsv13.TLSState;

import ru.ispras.telecomtesk.tlsv13.ConvertTypes;
import ru.ispras.telecomtesk.tlsv13.DebugInfo;

public class TLStrafficKey
{
	byte[] key;
	byte[] iv;
	long sec_num;

	public TLStrafficKey() {
		key = null; iv = null;
		sec_num = 0;
	}
	public TLStrafficKey(int keyLength, int ivLength) {
		key = new byte[keyLength]; iv = new byte[ivLength];
		sec_num = 0;
	}
	public TLStrafficKey(byte[] key, byte[] iv) {
		this.key = key; this.iv = iv;
		sec_num = 0;
	}

	public byte[] getKey(){
		return key;
	}

	public byte[] getNonce(){
		int i, p;
		byte[] nonce = new byte[iv.length];
		byte[] num = ConvertTypes.hton(sec_num);
		p = (iv.length > num.length) ? iv.length - num.length : 0;
		if(iv.length > num.length) {
			p = iv.length - num.length;
			for(i=0; i<p; i++) {
				nonce[i] = (byte)(0x00 ^ iv[i]);
			}
			for(i=0; i<num.length; i++,p++) {
				nonce[p] = (byte)(num[i] ^ iv[p]);
			}
		} else {
			p = num.length - iv.length;
			for(i=0; i<iv.length; i++) {
				nonce[i] = (byte)(num[p+i] ^ iv[i]);
			}
		}
		
		return nonce;
	}
	public long getSequenceNumber(){
		return sec_num;
	}
	public void setSequenceNumber(long sec_num){
		this.sec_num = sec_num;
	}
	public void incrementSequenceNumber(){
		sec_num ++;
	}
	public void decrementSequenceNumber(){
		sec_num --;
	}
		
	public void print(int debugLevel) {
		DebugInfo.printMessage(debugLevel,"TLStrafficKey Sequence number " + this.sec_num);
		DebugInfo.printDump(debugLevel, this.key, "Key: ");
		DebugInfo.printDump(debugLevel, this.iv, "IV: ");
	}

	public TLStrafficKey clone() {
		byte[] newkey = new byte[this.key.length];
		byte[] newiv = new byte[this.iv.length];
		System.arraycopy(this.key, 0, newkey, 0, this.key.length);
		System.arraycopy(this.iv, 0, newiv, 0, this.iv.length);
		TLStrafficKey res = new TLStrafficKey(newkey, newiv);
		res.sec_num = this.sec_num;
		return res;
	}

}
