package ru.ispras.telecomtesk.tlsv13.TLSspecification;

import ru.ispras.telecomtesk.tlsv13.DebugInfo;
import ru.ispras.telecomtesk.tlsv13.RecordProtocol.ChangeCipher;
import ru.ispras.telecomtesk.tlsv13.TLSAlgorithms.HandshakeMessageType;
import ru.ispras.telecomtesk.tlsv13.TLSState.MessageDirection;
import ru.ispras.telecomtesk.tlsv13.TLSState.TLSScenarioState;
import ru.ispras.telecomtesk.tlsv13.TLSclientSpecification.TLSClientSpec;
import ru.ispras.telecomtesk.tlsv13.TLSserverSpecification.TLSServerSpec;

public class CheckMessageOrder
{
    public static boolean checkOrder(HandshakeMessageType type, TLSServerSpec sp, MessageDirection dirc) {
    	boolean res = true;

		DebugInfo.printMessage(4,"CheckMessageOrder::checkOrder:");
		DebugInfo.printMessage(4, "HandshakeMessage: " + type + " " + dirc);
    	switch(type) {
    	case CLIENT_HELLO: 
        	if( (dirc == MessageDirection.SERVER_SEND) ||
        		(dirc == MessageDirection.CLIENT_RECEIVE) ||
        		( (sp.currentState != TLSScenarioState.CONNECTED) && 
        		  (sp.currentState != TLSScenarioState.AFTER_ServerHelloRetry) )	
        	  ) {
        		DebugInfo.printMessage(2,"===Error! CheckMessageOrder::checkOrder: failed for type: " + type);
        		res = false;
        	}
        	break;
    	case SERVER_HELLO: 
        	if( (dirc == MessageDirection.CLIENT_SEND) ||
        		(dirc == MessageDirection.SERVER_RECEIVE) ||
        		( (sp.currentState != TLSScenarioState.AFTER_ClientHELLO) &&
        		  (sp.currentState != TLSScenarioState.AFTER_ClientHELLOResume) )
      	      ) {
        		DebugInfo.printMessage(2,"===Error! CheckMessageOrder::checkOrder: failed for type: " + type);
        		res = false;
        	}
        	break;
    	case ENCRYPTED_EXTENSIONS: 
        	if( (dirc == MessageDirection.CLIENT_SEND) ||
        		(dirc == MessageDirection.SERVER_RECEIVE) ||
        		(sp.currentState != TLSScenarioState.AFTER_ServerHello) ) {
        		DebugInfo.printMessage(2,"===Error! CheckMessageOrder::checkOrder: failed for type: " + type);
        		res = false;
        	}
        	break;
    	case CERTIFICATE_REQUEST:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
           		res = false;
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
    			if( (!sp.isHandshakeCompleted && (sp.currentState != TLSScenarioState.AFTER_EncryptedExtensions)) || 
    				(sp.isHandshakeCompleted && !sp.isClientPostAuth) )
                	res = false;
            	break;
    		}
        	break;
    	case CERTIFICATE:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
    			if((sp.isGetCertificateRequest == false) || 
    				(sp.currentState != TLSScenarioState.AFTER_ServerFinished)) {
            		res = false;
    			}
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
            	if( ((sp.currentState != TLSScenarioState.AFTER_EncryptedExtensions) && 
                	(sp.currentState != TLSScenarioState.AFTER_CertificateRequest)) || 
                	(!sp.isServerMustSendCertificate) ) {
                		DebugInfo.printMessage(2,"===Error! CheckMessageOrder::checkOrder: failed for type: " + type);
                		DebugInfo.printMessage(2,"currentState: " + sp.currentState);
                		DebugInfo.printMessage(2,"isServerMustSendCertificate: " + sp.isServerMustSendCertificate);
                		res = false;
                	}
            	break;
    		}
        	break;
    	case CERTIFICATE_VERIFY:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
    			if(sp.currentState != TLSScenarioState.AFTER_ClientCertificate) {
            		res = false;
    			}
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
            	if(sp.currentState != TLSScenarioState.AFTER_ServerCertificate) {
               		res = false;
               	}
            	break;
    		}
        	break;
    	case END_OF_EARLY_DATA:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
    			if((!sp.mustSendEndOfEarlyData) || (sp.currentState != TLSScenarioState.AFTER_ServerFinished)) {
            		res = false;
    			}
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
             	res = false;
            	break;
    		}
        	break;
    	case FINISHED:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
    			if((sp.currentState != TLSScenarioState.AFTER_ServerFinished) && 
    				(sp.currentState != TLSScenarioState.AFTER_ClientCertificateVerify)){
            		res = false;
    			}
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
    			if((sp.currentState != TLSScenarioState.AFTER_EncryptedExtensions) && 
        			(sp.currentState != TLSScenarioState.AFTER_ServerCertificateVerify)){
                	res = false;
        		}
            	break;
    		}
        	break;
    	case KEY_UPDATE:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
    			if(!sp.isHandshakeCompleted) 
    				res = false;
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
    			if(!sp.afterServerFinished)
                	res = false;
            	break;
    		}
        	break;
    	case NEW_SESSION_TICKET:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
           		res = false;
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
    			if(!sp.isHandshakeCompleted)
                	res = false;
            	break;
    		}
        	break;
        default: 
    		DebugInfo.printMessage(1,"===Error! CheckMessageOrder::checkOrder: unknown HandshakeMessage type: " + type);
        	res = false;
    	}

		DebugInfo.printMessage(4,"CheckMessageOrder::checkOrder: return " + res);
    	return res;
    }
    	
    public static boolean checkOrder(HandshakeMessageType type, TLSClientSpec sp, MessageDirection dirc) {
    	boolean res = true;

		DebugInfo.printMessage(4,"CheckMessageOrder::checkOrder:");
		DebugInfo.printMessage(4, "HandshakeMessage: " + type + " " + dirc);
    	switch(type) {
    	case CLIENT_HELLO: 
        	if( (dirc == MessageDirection.CLIENT_RECEIVE) ||
        		(dirc == MessageDirection.SERVER_SEND) ||
        		( (sp.currentState != TLSScenarioState.CONNECTED) &&
        		  (sp.currentState != TLSScenarioState.AFTER_ServerHelloRetry) )
      	      ) {
        		DebugInfo.printMessage(2,"===Error! CheckMessageOrder::checkOrder: failed for type: " + type);
        		res = false;
        	}
        	break;
    	case SERVER_HELLO: 
        	if( (dirc == MessageDirection.CLIENT_SEND) ||
        		(dirc == MessageDirection.SERVER_RECEIVE) ||
        		( (sp.currentState != TLSScenarioState.AFTER_ClientHELLO) &&
        		  (sp.currentState != TLSScenarioState.AFTER_ClientHELLOResume) )
      	      ) {
        		DebugInfo.printMessage(2,"===Error! CheckMessageOrder::checkOrder: failed for type: " + type);
        		res = false;
        	}
        	break;
    	case ENCRYPTED_EXTENSIONS: 
        	if( (dirc == MessageDirection.CLIENT_SEND) ||
        		(dirc == MessageDirection.SERVER_RECEIVE) ||
        		(sp.currentState != TLSScenarioState.AFTER_ServerHello) ) {
        		DebugInfo.printMessage(2,"===Error! CheckMessageOrder::checkOrder: failed for type: " + type);
        		res = false;
        	}
        	break;
    	case CERTIFICATE_REQUEST:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
           		res = false;
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
    			if( (!sp.isHandshakeCompleted && (sp.currentState != TLSScenarioState.AFTER_EncryptedExtensions)) || 
    				(sp.isHandshakeCompleted && !sp.isClientPostAuth) )
                	res = false;
            	break;
    		}
        	break;
    	case CERTIFICATE:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
    			if((sp.isGetCertificateRequest == false) || 
    				(sp.currentState != TLSScenarioState.AFTER_ServerFinished)) {
            		res = false;
    			}
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
            	if( ( (sp.currentState != TLSScenarioState.AFTER_EncryptedExtensions) && 
                	  (sp.currentState != TLSScenarioState.AFTER_CertificateRequest) ) || 
                	(!sp.isServerMustSendCertificate) ) {
                		DebugInfo.printMessage(2,"===Error! CheckMessageOrder::checkOrder: failed for type: " + type);
                		DebugInfo.printMessage(2,"currentState: " + sp.currentState);
                		DebugInfo.printMessage(2,"isServerMustSendCertificate: " + sp.isServerMustSendCertificate);
                		res = false;
                	}
            	break;
    		}
        	break;
    	case CERTIFICATE_VERIFY:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
    			if(sp.currentState != TLSScenarioState.AFTER_ClientCertificate) {
            		res = false;
    			}
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
            	if(sp.currentState != TLSScenarioState.AFTER_ServerCertificate) {
               		res = false;
               	}
            	break;
    		}
        	break;
    	case END_OF_EARLY_DATA:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
    			if((!sp.mustSendEndOfEarlyData) || (sp.currentState != TLSScenarioState.AFTER_ServerFinished)) {
            		res = false;
    			}
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
             	res = false;
            	break;
    		}
        	break;
    	case FINISHED:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
    			if((sp.currentState != TLSScenarioState.AFTER_ServerFinished) && 
    	    		(sp.currentState != TLSScenarioState.AFTER_ClientCertificate) && 
    				(sp.currentState != TLSScenarioState.AFTER_ClientCertificateVerify)){
            		res = false;
    			}
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
    			if((sp.currentState != TLSScenarioState.AFTER_EncryptedExtensions) && 
        			(sp.currentState != TLSScenarioState.AFTER_ServerCertificateVerify)){
                	res = false;
        		}
            	break;
    		}
        	break;
    	case KEY_UPDATE:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
    			if(!sp.isHandshakeCompleted) 
    				res = false;
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
    			if(!sp.afterServerFinished)
                	res = false;
            	break;
    		}
        	break;
    	case NEW_SESSION_TICKET:
    		switch(dirc) {
    		case CLIENT_SEND:
    		case SERVER_RECEIVE:
           		res = false;
    			break;
    		case SERVER_SEND: 
    		case CLIENT_RECEIVE:
    			if(!sp.isHandshakeCompleted)
                	res = false;
            	break;
    		}
        	break;
        default: 
    		DebugInfo.printMessage(1,"===Error! CheckMessageOrder::checkOrder: unknown HandshakeMessage type: " + type);
        	res = false;
    	}

		DebugInfo.printMessage(4,"CheckMessageOrder::checkOrder: return " + res);
    	return res;
    }
    	
    public static boolean checkOrder(byte type, TLSServerSpec sp, MessageDirection dirc) {
    	return checkOrder(HandshakeMessageType.valueOf(type), sp, dirc);
    }
    
    public static boolean checkOrder(byte type, TLSClientSpec sp, MessageDirection dirc) {
    	return checkOrder(HandshakeMessageType.valueOf(type), sp, dirc);
    }
    
    public static boolean checkOrder(ChangeCipher msg, TLSServerSpec sp, MessageDirection dirc) {
		return checkOrder(msg,sp.currentState,dirc);
    }
    
    public static boolean checkOrder(ChangeCipher msg, TLSClientSpec sp, MessageDirection dirc) {
		return checkOrder(msg,sp.currentState,dirc);
    }
    
    public static boolean checkOrder(ChangeCipher msg, TLSScenarioState currentState, MessageDirection dirc) {
    	boolean res = true;
		DebugInfo.printMessage(4,"CheckMessageOrder::checkOrder:ChangeCipher");
		switch(dirc) {
		case CLIENT_SEND:
		case SERVER_RECEIVE:
			if((currentState == TLSScenarioState.DISCONNECTED) ||
				(currentState == TLSScenarioState.CONNECTED) ||
				(currentState == TLSScenarioState.AFTER_ClientFinished)){
        		res = false;
			}
			break;
		case SERVER_SEND: 
		case CLIENT_RECEIVE:
			if((currentState == TLSScenarioState.DISCONNECTED) || 
				(currentState == TLSScenarioState.CONNECTED) || 
    			(currentState == TLSScenarioState.AFTER_ServerFinished)){
            	res = false;
    		}
        	break;
		}
		return res;
    }
}
