package ru.ispras.telecomtesk.tlsv13.TestsServer.CheckTlsConnection;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.security.Key;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.MessageDigest;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.Signature;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.security.spec.ECGenParameterSpec;
import java.security.spec.ECPoint;

import sun.security.ec.ECParameters;

import javax.crypto.Cipher;
import javax.crypto.Mac;
import javax.crypto.interfaces.DHPublicKey;
import javax.crypto.spec.DHParameterSpec;
import javax.crypto.spec.GCMParameterSpec;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import javax.xml.bind.DatatypeConverter;

import ru.ispras.telecomtesk.tlsv13.TLSState.*;
import ru.ispras.telecomtesk.tlsv13.TLSserverSpecification.TLSServerSpec;
import ru.ispras.telecomtesk.tlsv13.HandshakeMessages.*;
import ru.ispras.telecomtesk.tlsv13.HandshakeMessages.TLSExtensions.*;
import ru.ispras.telecomtesk.tlsv13.TLSSocket.*;
import ru.ispras.telecomtesk.tlsv13.RecordProtocol.*;
import ru.ispras.telecomtesk.tlsv13.TLSAlgorithms.*;
import ru.ispras.telecomtesk.tlsv13.LoginInfo;
import ru.ispras.telecomtesk.tlsv13.DebugInfo;
import java.security.MessageDigest;

class CreateClientHello2 {
	protected String targetServer;
	protected int targetPort;
	protected String socketTimeout;
	public LoginInfo LI = new LoginInfo();
	public TLSConnection current_connection;
	public TLSServerSpec target = new TLSServerSpec();
	public InetAddress hostIP;
	public String applicationDataString = null;

	public CreateClientHello2(LoginInfo scenProp) {
		LI = scenProp;
		DebugInfo.debugLevel = 7;
		DebugInfo.maxDumpSize = 4000;

		targetServer = scenProp.getExistingValue("server");
		if (targetServer == null) {
			throw new IllegalArgumentException("Server not specified");
		}

		String targetPortString = scenProp.getExistingValue("portTLS");
		if (targetPortString == null) {
			throw new IllegalArgumentException("Server port not specified");
		}
		targetPort = Integer.parseInt(targetPortString);

		applicationDataString = scenProp.getValue("applicationData", "Test message");

		socketTimeout = scenProp.getValue("timeout");

		TLSConnectionSelector sel = new TLSConnectionSelector(null,null,0,0);
		current_connection = new TLSConnection(sel,ConnectionRole.CLIENT);
		// by default current_connection.getPendingState().getParams().isServer = false; 
	
		try {
			hostIP = InetAddress.getLocalHost();
		} catch(UnknownHostException err) {
			throw new RuntimeException( "Failed to initialize scenario",err);
		}

	}

	public void ClientHello() {
		DebugInfo.printMessageWithSeparator(1,"scenario ClientHello start");
		
	
		ArrayList<RecordLayer> response;
		short[] cs = {TLSCipherSuite.TLS_AES_128_GCM_SHA256.getValue(),
					  TLSCipherSuite.TLS_AES_128_CCM_SHA256.getValue(),
					TLSCipherSuite.TLS_RSA_WITH_AES_128_CBC_SHA.getValue(),
					TLSCipherSuite.TLS_RSA_WITH_AES_256_CBC_SHA256.getValue()
					 };
		Hello msg;
	SignatureAndHashAlgorithm[] sah12 = {
			new SignatureAndHashAlgorithm(HashAlgorithm.SHA1, SignatureAlgorithm.RSA),
			new SignatureAndHashAlgorithm(HashAlgorithm.MD5, SignatureAlgorithm.RSA),
			new SignatureAndHashAlgorithm(HashAlgorithm.SHA256, SignatureAlgorithm.RSA)
			};
	SignatureAndHashAlgorithmExt12 hl_ext_sah12 = new SignatureAndHashAlgorithmExt12(sah12);

		SignatureScheme[] sah = {SignatureScheme.RSA_PKCS1_SHA256, SignatureScheme.ECDSA_SECP256R1_SHA256};
		SignatureAndHashAlgorithmExt hl_ext_sah = new SignatureAndHashAlgorithmExt(sah);
		SignatureAndHashAlgorithmExt hl_ext_sah2 = new SignatureAndHashAlgorithmExt(sah);

		NamedGroup[] sg = {NamedGroup.FFDHE2048, NamedGroup.FFDHE6144, NamedGroup.SECP256R1};
		SupportedGroupsExt hl_ext_sg = new SupportedGroupsExt(sg);

		NamedGroup[] sendgrp = {NamedGroup.FFDHE2048, NamedGroup.FFDHE6144};
		current_connection.hskParams.generateClientKeyPairList(sendgrp);

		ProtocolVersion svdata = new ProtocolVersion(0x03,0x04);
		TLSExtension hl_ext_sv = new SupportedVersionsExt(svdata);
/*
	    // If negotiating TLS 1.2 or 1.1, TLS 1.3 servers MUST set the last 8 bytes
	    // of their Random value to the bytes:
	    byte[] randomDowngradeProtectionTLS12 = DatatypeConverter.parseHexBinary(
	    		"CF21AD74E59A6111BE1D8C021E65B891" +
	    	    "C2A211167ABB8C5E444F574E47524401");
//		TLS13SpecValues.randomHelloRetryRequest;

//		ECGenParameterSpec ecsp = new ECGenParameterSpec("secp256r1");


//	    KeyPairGenerator keyGen = KeyPairGenerator.getInstance("EC");
//	    keyGen.initialize(new ECGenParameterSpec("secp256r1"), new SecureRandom());
//	    KeyPair keyPair = keyGen.generateKeyPair();


//		KeyShareEntry kse = new KeyShareEntry(NamedGroup.SECP256R1, dhpk);
		TLSExtension hl_ext_ks = new KeyShareClientHello(current_connection.hskParams.clientKeyShareList);
//		TLSExtension hl_ext_ks = new KeyShareClientHello();

		byte[] dataext = {1,2,3,4,5};
		TLSExtension hl_ext_un = new TLSExtension(1020,dataext);
		TLSExtension hl_ext_un2 = new TLSExtension(1020,dataext);

		// create Hello
		DebugInfo.printMessage(1,"create Hello");
//		msg = new Hello(cs);
		msg = new Hello(TLS13SpecValues.randomHelloRetryRequest,null,cs);
//		msg = new Hello(randomDowngradeProtectionTLS12,null,cs);
//		msg.addExtension(hl_ext_sah12);
		msg.addExtension(hl_ext_sv);
		msg.addExtension(hl_ext_sah);
		msg.addExtension(hl_ext_sg);
		msg.addExtension(hl_ext_ks);
		
		DebugInfo.printMessage(1,"print Hello");
		msg.print();

		current_connection.updateState(msg);
*/
		byte[] encPkt = DatatypeConverter.parseHexBinary(
"be3e348ff7dd15cc03de1dee53ec78708942d3"
				);

		byte[] decPkt = DatatypeConverter.parseHexBinary(
	"0800001C001A000A00160014001D0017" + 
	"00180019001E01000101010201030104" + 
	"1600000000000000000000000000000000");

		TLSSecurityParameters sp = new TLSSecurityParameters();
		sp.setAlgorithmParams(TLSCipherSuite.TLS_AES_128_GCM_SHA256);
		String scl = "27C98D287336C4BD10375B5DDF7452C8AA532D9842E81300CB75B1625A602A67";
		String ssr = "9AA9E48833E233C1A6BC900275D222EEAA9E782FB1F3BCB9C269B6E369B6FF95";
		sp.client_handshake_traffic_secret = DatatypeConverter.parseHexBinary(scl);
		sp.server_handshake_traffic_secret = DatatypeConverter.parseHexBinary(ssr);
		String acl = "6577CE11FF2C12879617608A39B95FB02599C74A4A4EAE8C8C88F212897B766D";
		String asr = "368283C6BF9659EF10B40934EF4B42DD681C3B2776EF1E103FC97269F497D0E8";
		sp.client_application_traffic_secret = DatatypeConverter.parseHexBinary(acl);
		sp.server_application_traffic_secret = DatatypeConverter.parseHexBinary(asr);

		try {
			sp.createHandshakeTrafficKeys();
			sp.printDump(1);
			sp.createClientApplicationTrafficKeys();
			sp.createServerApplicationTrafficKeys();
			sp.printDump(1);
			/*
		byte[] additional_data = {0x17,0x03,0x03,0x0d,(byte)0x8a};
		int taglen = 128;
		byte[] iv = DatatypeConverter.parseHexBinary("6205F69D1E93C52B9954F337");
		byte[] aeskey = DatatypeConverter.parseHexBinary("5276BC44859BE0BB3072F9E4C2F977F4");
		TLStrafficKey tmpkey = new TLStrafficKey(aeskey, iv);
				
		SecretKeySpec sk = new SecretKeySpec(aeskey, "AES");
		Cipher cobj1 = Cipher.getInstance("AES/GCM/NoPadding");

		byte[] num = new byte[iv.length];
		byte[] nonce = new byte[iv.length];
		for(int i=0; i<num.length; i++) num[i] = 0;
//		num[num.length-1] = 1;
		for(int i=0; i<iv.length; i++) {
			nonce[i] = (byte)(num[i] ^ iv[i]);
		}
		DebugInfo.printDump(1, num, "   num: ");
		DebugInfo.printDump(1, nonce, "   nonce: ");
		DebugInfo.printDump(1, iv, "   iv: ");
		DebugInfo.printDump(1, tmpkey.getNonce(), "   nonce2: ");
		
		GCMParameterSpec gcmsp = new GCMParameterSpec(taglen, nonce);
		cobj1.init(Cipher.ENCRYPT_MODE, sk, gcmsp);
		cobj1.updateAAD(additional_data);
		byte[] enc = cobj1.doFinal(decPkt); 

		DebugInfo.printDump(1, enc, "   enc: ");

		Cipher cobj2 = Cipher.getInstance("AES/GCM/NoPadding");
		cobj2.init(Cipher.DECRYPT_MODE, sk, gcmsp);
		cobj2.updateAAD(additional_data);
		byte[] plain = cobj2.doFinal(encPkt); 
		*/
		sp.serverKey.incrementSequenceNumber();
		byte[] plain = 	sp.decrypt(encPkt,ConnectionRole.CLIENT);
		DebugInfo.printDump(1, plain, "   plain: ");
		} catch (Exception e) {DebugInfo.printMessage(1,"error" + e);}
		
		return;
	}
	
	

	public static void main(String[] args) {
		Date dt1 = new Date();
		System.out.println(dt1.toString());
//		String fileName = "connectInfo_mail.txt";
//		String fileName = "connectInfo_http.txt";
		String fileName = "connectInfo_java.txt";
		if (args.length > 0 )
			fileName = args[0];
		start(fileName);
		Date dt2 = new Date();
		System.out.println(dt2.toString());
	}
	
	
	public static void start(String fileName) {
		
		LoginInfo LI = new LoginInfo();
		LI.readFromFile(fileName);

		CreateClientHello2 scen = new CreateClientHello2(LI);
		scen.ClientHello();
	}
}