package ru.ispras.telecomtesk.tlsv13.TestsServer.CheckTlsConnection;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.lang.invoke.MethodHandles;
import java.math.BigInteger;
import java.security.Key;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.spec.ECGenParameterSpec;
import java.security.spec.ECPoint;

import sun.security.ec.ECParameters;
import java.util.Date;
import java.util.Random;

import javax.crypto.interfaces.DHPublicKey;
import javax.crypto.spec.DHParameterSpec;
import javax.xml.bind.DatatypeConverter;

import ru.ispras.telecomtesk.tlsv13.TLSState.*;
import ru.ispras.telecomtesk.tlsv13.TLSserverSpecification.TLSServerSpec;
import ru.ispras.telecomtesk.tlsv13.HandshakeMessages.*;
import ru.ispras.telecomtesk.tlsv13.HandshakeMessages.TLSExtensions.*;
import ru.ispras.telecomtesk.tlsv13.TLSSocket.*;
import ru.ispras.telecomtesk.tlsv13.RecordProtocol.*;
import ru.ispras.telecomtesk.tlsv13.TLSAlgorithms.*;
import ru.ispras.telecomtesk.tlsv13.LoginInfo;
import ru.ispras.telecomtesk.tlsv13.DebugInfo;

class TestServerHelloRetry {
	protected String targetServer;
	protected int targetPort;
	protected String socketTimeout;
	public LoginInfo LI = new LoginInfo();
	public TLSConnection current_connection;
	public TLSServerSpec target = new TLSServerSpec();
	public InetAddress hostIP;
	public String applicationDataString = "Test message";
	public static long testCounter = 0;
	public static PrintWriter logFile;

	public TestServerHelloRetry(LoginInfo scenProp) {
		LI = scenProp;
		DebugInfo.debugLevel = 5;
		DebugInfo.maxDumpSize = 400;

		targetServer = scenProp.getExistingValue("server");
		if (targetServer == null) {
			throw new IllegalArgumentException("Server not specified");
		}

		String targetPortString = scenProp.getExistingValue("portTLS");
		if (targetPortString == null) {
			throw new IllegalArgumentException("Server port not specified");
		}
		targetPort = Integer.parseInt(targetPortString);

		applicationDataString = scenProp.getValue("applicationData", "Test message");

		socketTimeout = scenProp.getValue("timeout");

		TLSConnectionSelector sel = new TLSConnectionSelector(null,null,0,0);
		current_connection = new TLSConnection(sel,ConnectionRole.CLIENT);
		// by default current_connection.getPendingState().getParams().isServer = false; 
	
		try {
			hostIP = InetAddress.getLocalHost();
		} catch(UnknownHostException err) {
			throw new RuntimeException( "Failed to initialize scenario",err);
		}

		target.sock = new TLSSocket(targetServer, targetPort, Integer.parseInt(socketTimeout));
	}

	public void connect() {
		DebugInfo.printMessageWithSeparator(1,"scenario connect start");
		try {
			target.connect(current_connection);
		}
		catch (IOException err) {
			System.out.println("scenario Connect failed" + err);
		}
	}

	public void ClientHello() {
		if(target.getCurrentState() != TLSScenarioState.CONNECTED)
			return;
		DebugInfo.printMessageWithSeparator(1,"scenario ClientHello start");
		short[] cs = {TLSCipherSuite.TLS_AES_128_GCM_SHA256.getValue(),
					TLSCipherSuite.TLS_RSA_WITH_AES_128_CBC_SHA.getValue(),
					TLSCipherSuite.TLS_RSA_WITH_AES_256_CBC_SHA256.getValue()
					 };
		Hello msg;
	SignatureAndHashAlgorithm[] sah12 = {
			new SignatureAndHashAlgorithm(HashAlgorithm.SHA1, SignatureAlgorithm.RSA),
			new SignatureAndHashAlgorithm(HashAlgorithm.MD5, SignatureAlgorithm.RSA),
			new SignatureAndHashAlgorithm(HashAlgorithm.SHA256, SignatureAlgorithm.RSA)
			};
	SignatureAndHashAlgorithmExt12 hl_ext_sah12 = new SignatureAndHashAlgorithmExt12(sah12);

		SignatureScheme[] sah = {SignatureScheme.RSA_PKCS1_SHA256, SignatureScheme.ECDSA_SECP256R1_SHA256};
		SignatureAndHashAlgorithmExt hl_ext_sah = new SignatureAndHashAlgorithmExt(sah);
		SignatureAndHashAlgorithmExt hl_ext_sah2 = new SignatureAndHashAlgorithmExt(sah);

		NamedGroup[] sg = {NamedGroup.FFDHEtest, NamedGroup.FFDHE2048};
		SupportedGroupsExt hl_ext_sg = new SupportedGroupsExt(sg);

		NamedGroup[] sendgrp = {NamedGroup.FFDHEtest};
		current_connection.hskParams.generateClientKeyPairList(sendgrp);

		ProtocolVersion[] svdata = {new ProtocolVersion(0x03,0x04),new ProtocolVersion(0x03,0x03)};
		TLSExtension hl_ext_sv = new SupportedVersionsExt(svdata);

	    // If negotiating TLS 1.2 or 1.1, TLS 1.3 servers MUST set the last 8 bytes
	    // of their Random value to the bytes:
	    byte[] randomDowngradeProtectionTLS12 = DatatypeConverter.parseHexBinary(
	    		"CF21AD74E59A6111BE1D8C021E65B891" +
	    	    "C2A211167ABB8C5E444F574E47524401");
//		TLS13SpecValues.randomHelloRetryRequest;

//		ECGenParameterSpec ecsp = new ECGenParameterSpec("secp256r1");


//	    KeyPairGenerator keyGen = KeyPairGenerator.getInstance("EC");
//	    keyGen.initialize(new ECGenParameterSpec("secp256r1"), new SecureRandom());
//	    KeyPair keyPair = keyGen.generateKeyPair();


//		KeyShareEntry kse = new KeyShareEntry(NamedGroup.SECP256R1, dhpk);
		TLSExtension hl_ext_ks = new KeyShareClientHello(current_connection.hskParams.clientKeyShareList);
//		TLSExtension hl_ext_ks = new KeyShareClientHello();

		byte[] dataext = {1,2,3,4,5};
		TLSExtension hl_ext_un = new TLSExtension(1020,dataext);
		TLSExtension hl_ext_un2 = new TLSExtension(1020,dataext);

		// create Hello
		DebugInfo.printMessage(1,"create Hello");
		byte[] dataId = {1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6,7,8};
		msg = new Hello(dataId,cs);
//		msg = new Hello(TLS13SpecValues.randomHelloRetryRequest,null,cs);
//		msg = new Hello(randomDowngradeProtectionTLS12,null,cs);
//		msg.addExtension(hl_ext_sah12);
		msg.addExtension(hl_ext_sv);
		msg.addExtension(hl_ext_sah);
		msg.addExtension(hl_ext_sg);
		msg.addExtension(hl_ext_ks);
		
		DebugInfo.printMessage(1,"print Hello");
		msg.print();
		
		Exception err = msgSetContentAndSend(msg,null,"ClientHello");
		if(err != null) {
			throw new RuntimeException( "scenario ClientHello: msgSetContentAndSend failed",err);
		}
		
		return;
	}
	
	public void ClientHello2() {
		DebugInfo.printMessageWithSeparator(4,"!!!scenario ClientHello2 try. " + target.currentState);
		if(target.getCurrentState() != TLSScenarioState.AFTER_ServerHelloRetry)
			return;
		DebugInfo.printMessageWithSeparator(1,"scenario ClientHello2 start");
		short[] cs = {TLSCipherSuite.TLS_AES_128_GCM_SHA256.getValue(),
				TLSCipherSuite.TLS_RSA_WITH_AES_128_CBC_SHA.getValue(),
				TLSCipherSuite.TLS_RSA_WITH_AES_256_CBC_SHA256.getValue()
				 };

		SignatureScheme[] sah = {SignatureScheme.RSA_PKCS1_SHA256, SignatureScheme.ECDSA_SECP256R1_SHA256};
		SignatureAndHashAlgorithmExt hl_ext_sah = new SignatureAndHashAlgorithmExt(sah);

		NamedGroup[] sg = {NamedGroup.FFDHEtest, NamedGroup.FFDHE2048};
		SupportedGroupsExt hl_ext_sg = new SupportedGroupsExt(sg);

		NamedGroup[] sendgrp = {NamedGroup.FFDHE2048};
		current_connection.hskParams.generateClientKeyPairList(sendgrp);

		ProtocolVersion[] svdata = {new ProtocolVersion(0x03,0x04),new ProtocolVersion(0x03,0x03)};
		TLSExtension hl_ext_sv = new SupportedVersionsExt(svdata);
		TLSExtension hl_ext_ks = new KeyShareClientHello(current_connection.hskParams.clientKeyShareList);
		byte[] dataId = {1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6,7,8};
		Hello msg = new Hello(dataId,cs);
		msg.addExtension(hl_ext_sv);
		msg.addExtension(hl_ext_sah);
		msg.addExtension(hl_ext_sg);
		if(target.needCookie) {
			CookieExt hl_ext_ck = new CookieExt(current_connection.hskParams.remoteCookie);
			msg.addExtension(hl_ext_ck);
		}
		msg.addExtension(hl_ext_ks);
		
		DebugInfo.printMessage(1,"print Hello2");
		msg.print();
		
		Exception err = msgSetContentAndSend(msg,null,"ClientHello2");
		if(err != null) {
			throw new RuntimeException( "scenario ClientHello2: msgSetContentAndSend failed",err);
		}
		
		return;
	}
	
	
	public void Finished() {
		if(!target.getCurrentState().connected)
			return;
		DebugInfo.printMessageWithSeparator(1,"scenario Finished start");
		Finished msg = null;
		try {
			msg = new Finished(current_connection, MessageDirection.getMessageDirection(current_connection.localRole, TLSDirection.SEND)); 
		}
		catch (Exception err) {
			DebugInfo.printMessage(1,"===Warning! create new Finished message failed");
		}
		
		Exception err = msgSetContentAndSend(new RecordLayer(msg),msg,null,"Finished");
		if(err != null) {
			throw new RuntimeException( "scenario Finished: msgSetContentAndSend failed",err);
		}
		
		return;
	}
	
	public void ChangeCipher() {
		if(!target.getCurrentState().connected)
			return;
		DebugInfo.printMessageWithSeparator(1,"scenario ChangeCipher start");
		ChangeCipher msg = new ChangeCipher();
		
		RecordLayer tlsmsg = new RecordLayer(TLSContentType.CHANGE_CIPHER_SPEC, msg);
		tlsmsg.initContent(1);
		tlsmsg.content[0] = msg.toByteArray();
		target.mayNoResponse = true;

		Exception err = msgSend(tlsmsg, "ChangeCipher");
		if(err != null) {
			throw new RuntimeException( "scenario ChangeCipher: msgSend failed",err);
		}
		
		return;
	}
	
	public void ApplicationData() {
		if(!target.getCurrentState().connected)
			return;
		
		DebugInfo.printMessageWithSeparator(1,"scenario ApplicationData start");
		System.out.println(applicationDataString);
		ApplicationData msg = new ApplicationData(applicationDataString.getBytes());

		Exception err = msgSetContentAndSend(new RecordLayer(msg),msg,null,"ApplicationData");
		if(err != null) {
			throw new RuntimeException( "scenario ApplicationData: msgSetContentAndSend failed",err);
		}
		
		target.disconnectWithSendAlert = 0;
		target.DisconnectWithAlert(current_connection);

		return;
	}
	
	
	

	public Exception msgSetContent(Object msg, byte[] padding, String scenName) {
		return msgSetContent(null, new RecordLayer(msg), msg, padding, scenName);
	}
	
	public Exception msgSetContent(TLSContentType tp, Object msg, byte[] padding, String scenName) {
		return msgSetContent(tp, new RecordLayer(msg), msg, padding, scenName);
	}
	
	public Exception msgSetContent(RecordLayer rlmsg, Object msg, byte[] padding, String scenName) {
		return msgSetContent(null, rlmsg, msg, padding, scenName);
	}
	
	public Exception msgSetContent(TLSContentType tp, RecordLayer rlmsg, Object msg, byte[] padding, String scenName) {
		Exception res = null;
		
		try {
			DebugInfo.printMessage(1," --- Object --- ");
			rlmsg.setContent(msg,padding,current_connection);
			if(tp != null) {
				rlmsg.tls_type = tp;
				rlmsg.opaque_tls_type = tp;
			}
			DebugInfo.printDump(1, rlmsg.toByteArray(), scenName);
		}
		catch (Exception err) {
			DebugInfo.logtrace(logFile, "===Error! counter: " + testCounter + " scenario " + scenName);
			DebugInfo.logtrace(logFile, "===Error! scenario setContent Exception: " + err);
//			target.disconnect();
			return res;
		}
		return res;
	}
	
	
	public Exception msgSetContent(TLSContentType tp, RecordLayer rlmsg, byte[] msg, byte[] padding, String scenName) {
		Exception res = null;
		
		try {
			DebugInfo.printMessage(1," --- byte[] --- ");
			rlmsg.setContent(msg,padding,current_connection);
			if(tp != null) {
				rlmsg.tls_type = tp;
				rlmsg.opaque_tls_type = tp;
			}
			DebugInfo.printDump(1, rlmsg.toByteArray(), scenName);
		}
		catch (Exception err) {
			DebugInfo.logtrace(logFile, "===Error! counter: " + testCounter + " scenario " + scenName);
			DebugInfo.logtrace(logFile, "===Error! scenario setContent Exception: " + err);
//			target.disconnect();
			return res;
		}
		return res;
	}
	
	
	public Exception msgSetContentAndSend(Object msg, byte[] padding, String scenName) {
		return msgSetContentAndSend(new RecordLayer(msg), msg, padding, scenName);
	}
	
	public Exception msgSetContentAndSend(RecordLayer rlmsg, Object msg, byte[] padding, String scenName) {
		Exception res = null;
		
		try {
			rlmsg.setContent(msg,padding,current_connection);
			DebugInfo.printDump(5, rlmsg.toByteArray(), scenName);
//			if(msg instanceof HandshakeMessage)
//				DebugInfo.printDump(5, ((HandshakeMessage)msg).net_msg, scenName);
		}
		catch (Exception err) {
			DebugInfo.logtrace(logFile, "===Error! counter: " + testCounter + " scenario " + scenName);
			DebugInfo.logtrace(logFile, "===Error! scenario setContent Exception: " + err);
			target.disconnect();
			return res;
		}

		res = msgSend(rlmsg, scenName);
		
		return res;
	}
	
	
	public Exception msgSend(RecordLayer rlmsg, String scenName) {
		Exception res = null;
		
		try {
			ArrayList<RecordLayer> response = target.send(rlmsg,current_connection);
			if(null == response) {
				DebugInfo.logtrace(logFile, "===Error! counter: " + testCounter + " scenario " + scenName);
				DebugInfo.logtrace(logFile,"===Error! scenario send response null, may be decrypt failed");
				target.disconnectWithSendAlert = 80;
				target.DisconnectWithAlert(current_connection);
			}
			else if(target.disconnectWithSendAlert != -1) {
				target.DisconnectWithAlert(current_connection);
			}
		}
		catch (Exception err) {
			DebugInfo.logtrace(logFile, "===Error! counter: " + testCounter + " scenario " + scenName);
			DebugInfo.logtrace(logFile,"===Error! scenario send Exception: " + err);
			target.disconnect();	
			res = err;
		}
		
		return res;
	}
	
	
	public static void main(String[] args) {
//		String fileName = "connectInfo_mail.txt";
//		String fileName = "connectInfo_http.txt";
		String fileName = "connectInfo_java.txt";
		if (args.length > 0 )
			fileName = args[0];
		start(fileName);
	}
	
	
	public static void start(String fileName) {
		String currentClassName = MethodHandles.lookup().lookupClass().getSimpleName();
		try {
			logFile = new PrintWriter(new OutputStreamWriter(new FileOutputStream(currentClassName + "_errorlog.txt",true)));
			DebugInfo.printMessage(1,"open log file " + currentClassName + "_errorlog.txt");
		}
		catch (FileNotFoundException err) {
			DebugInfo.printMessage(1,"open log file Exception: " + err);
			return;
		}
		Date dt1 = new Date();
		DebugInfo.logtrace(logFile,"start test " + currentClassName + " " + dt1.toString());
		
		LoginInfo LI = new LoginInfo();
		LI.readFromFile(fileName);
		
		TestServerHelloRetry scen = new TestServerHelloRetry(LI);
		
		scen.connect();
		scen.ClientHello();
//		scen.ChangeCipher();
		scen.ClientHello2();
		scen.Finished();
		scen.ApplicationData();

		dt1 = new Date();
		DebugInfo.logtrace(logFile,"end test: " + dt1.toString() + " testCounter " + testCounter);
	}
}