package ru.ispras.telecomtesk.tlsv13;

import java.io.File;
import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.util.Enumeration;

import ru.ispras.telecomtesk.tlsv13.TLSAlgorithms.HashAlgorithm;
import ru.ispras.telecomtesk.tlsv13.TLSAlgorithms.SignatureAlgorithm;
import ru.ispras.telecomtesk.tlsv13.TLSAlgorithms.SignatureAndHashAlgorithm;
import ru.ispras.telecomtesk.tlsv13.TLSAlgorithms.SignatureScheme;
import ru.ispras.telecomtesk.tlsv13.TLSState.TLSHandshakeParameters;

public class TlsKeystore {
	
	public static KeyStore getKeyStore(String keystore, String keystorePasswd) 
	{
		KeyStore ks = null;
		if ((null != keystore) && (null != keystorePasswd)) {
			try {
				File t = new File(keystore);
				if (!t.exists()) {
					throw new RuntimeException("Could not find key manager file: " + keystore);
				}
				DebugInfo.printMessage(3,"Using keystore: " + keystore);
				ks = KeyStore.getInstance("JKS");
				ks.load(new FileInputStream(keystore), keystorePasswd.toCharArray());
			} catch (Exception e) {
				DebugInfo.printMessage(1,"===Error! TlsKeystore::getKeyStore: getKeyManagerArray Exception");
				e.printStackTrace();
			}
		}
		return ks;
	}

	public static Certificate getCertificate(KeyStore ks, SignatureAndHashAlgorithm[] sah, SignatureAlgorithm sig) 
	{
		Certificate cert = null;
		String certals = null;
		String sigalg = null;
		boolean certexist = false;
		
		if (null != ks) {
			try {
				Enumeration<String> certlst = ks.aliases();

				while(certlst.hasMoreElements()) {
					certals = certlst.nextElement();
					cert = ks.getCertificate(certals);
					DebugInfo.printMessage(3,"getCertificate: Certificate alias: " + certals + ", type: " + cert.getType());

					if((sah == null) || (sah.length == 0)) {
					DebugInfo.printMessage(3,"getCertificate: Return first certificate");
						if(sig != null  &&  cert.getType() == "X.509") {
							X509Certificate xc = (X509Certificate)cert;
							DebugInfo.printMessage(3,"getCertificate: Certificate SigAlgName: " + xc.getSigAlgName());
							SignatureAndHashAlgorithm certsah = getSigAlg(cert);
							if(certsah.signature != sig.getValue()) {
								continue;
							}
						}
						certexist = true;
						break;
					}
					else
						if(cert.getType() == "X.509") {
							X509Certificate xc = (X509Certificate)cert;
							DebugInfo.printMessage(3,"getCertificate: Certificate SigAlgName: " + xc.getSigAlgName());
							for(SignatureAndHashAlgorithm cursah : sah) {
								if(cursah != null) {
									if(sig != null  &&  cursah.signature != sig.getValue())
										continue;
									sigalg = HashAlgorithm.valueOf(cursah.hash) + "with" + SignatureAlgorithm.valueOf(cursah.signature);
									if(sigalg.equalsIgnoreCase(xc.getSigAlgName())) {
										certexist = true;
										break; 
									}
								}
							}
							if(certexist == true)
								break;
						}
				}
			} catch (Exception e) {
				DebugInfo.printMessage(1,"===Error! TlsKeystore::getCertificate: Exception");
				e.printStackTrace();
			}
		}
		if(certexist)
			return cert;
		else
			return null;
	}

	public static Certificate getCertificate(KeyStore ks) 
	{
		Certificate cert = null;
		String certals = null;
		
		if (null != ks) {
			try {
				Enumeration<String> certlst = ks.aliases();

				if(certlst.hasMoreElements()) {
					certals = certlst.nextElement();
					cert = ks.getCertificate(certals);
					DebugInfo.printMessage(3,"getCertificate: Certificate alias: " + certals + ", type: " + cert.getType());
				}
			} catch (Exception e) {
				DebugInfo.printMessage(1,"===Error! TlsKeystore::getCertificate: Exception");
				e.printStackTrace();
			}
		}
		return cert;
	}

	public static Certificate getCertificate(KeyStore ks, SignatureScheme ss) 
	{
		Certificate cert = null;
		String certals = null;
		String sigalg = null;
		boolean certexist = false;
		
		DebugInfo.printMessage(4,"getCertificate: SignatureScheme: " + ss);

		if ((null != ks)&&(null != ss)) {
			try {
				Enumeration<String> certlst = ks.aliases();

				while(certlst.hasMoreElements()) {
					certals = certlst.nextElement();
					cert = ks.getCertificate(certals);
					DebugInfo.printMessage(4,"getCertificate: Certificate alias: " + certals + ", type: " + cert.getType());

					if(cert.getType() == "X.509") {
						X509Certificate xc = (X509Certificate)cert;
						DebugInfo.printMessage(4,"getCertificate: Certificate SigAlgName: " + xc.getSigAlgName());
						sigalg = ss.getHashName() + "with" + ss.getSigName();
						if(sigalg.equalsIgnoreCase(xc.getSigAlgName())) {
							certexist = true;
							break; 
						}
					}
				}
			} catch (Exception e) {
				DebugInfo.printMessage(1,"===Error! TlsKeystore::getCertificate: Exception");
				e.printStackTrace();
			}
		}

		return certexist ? cert : null;
	}

	public static Certificate getCertificate(KeyStore ks, TLSHandshakeParameters hprm) 
	{
		Certificate cert = null;
		
		if ((null != ks)&&(null != hprm)&&(null != hprm.clientSignatureScheme)&&(0 != hprm.clientSignatureScheme.length)) {
			for(SignatureScheme ss : hprm.clientSignatureScheme) {
				cert = getCertificate(ks,ss);
				if (null != cert) {
					hprm.localCertType = ss;
					hprm.localCert = cert;
					break;
				}
			}
		}
		return cert;
	}

	public static SignatureAndHashAlgorithm getSigAlg(Certificate cert) 
	{
		SignatureAndHashAlgorithm alg = null;
		
		if (cert != null) {
			try {
				DebugInfo.printMessage(4,"getSigAlg: Certificate type: " + cert.getType());
				if(cert.getType() == "X.509") {
					X509Certificate xc = (X509Certificate)cert;
					String algname = xc.getSigAlgName();
					DebugInfo.printMessage(4,"getSigAlg: Certificate SigAlgName: " + algname);
					String[] alglst = algname.split("with");
					if((alglst != null) && (alglst.length == 2)) {
						try {
							alg = new SignatureAndHashAlgorithm(HashAlgorithm.valueOf(alglst[0]), SignatureAlgorithm.valueOf(alglst[1]));
						} catch (Exception e) {
							DebugInfo.printMessage(3,"===Warning! TlsKeystore::getSigAlg: bad Certificate SigAlgName");
						}
					}
					else
						DebugInfo.printMessage(3,"===Warning! TlsKeystore::getSigAlg: bad Certificate SigAlgName");
				}
			} catch (Exception e) {
				DebugInfo.printMessage(1,"===Error! TlsKeystore::getCertificateChain: Exception");
				e.printStackTrace();
			}
		}
		return alg;
	}

	public static PrivateKey getPrivateKeyCert(KeyStore ks, Certificate cert, String passwd) 
	{
		PrivateKey prkey = null;
		
		if ((null != ks) && (cert != null) && (passwd != null)) {
			try {
				prkey = (PrivateKey) ks.getKey(ks.getCertificateAlias(cert), passwd.toCharArray());
			} catch (Exception e) {
				DebugInfo.printMessage(1,"===Error! TlsKeystore::getCertificateChain: Exception");
				e.printStackTrace();
			}
		}
		return prkey;
	}

	public static Certificate[] getCertificateChain(KeyStore ks, Certificate cert) 
	{
		Certificate[] certchain = null;
		DebugInfo.printMessage(3,"TlsKeystore::getCertificateChain");
		
		if ((null != ks) && (cert != null)) {
			try {
				certchain = ks.getCertificateChain(ks.getCertificateAlias(cert));
				if(certchain != null)
					DebugInfo.printMessage(3,"TlsKeystore::getCertificateChain: length " + certchain.length);
				else
					DebugInfo.printMessage(3,"TlsKeystore::getCertificateChain: length 0");
			} catch (Exception e) {
				DebugInfo.printMessage(1,"===Error! TlsKeystore::getCertificateChain: Exception");
				e.printStackTrace();
			}
		}
		return certchain;
	}

	public static byte[][] getCertificateChainEncoded(KeyStore ks, Certificate cert) 
	{
		byte[][] chain = null;
		DebugInfo.printMessage(3,"TlsKeystore::getCertificateChainEncoded");
		
		if ((null != ks) && (cert != null)) {
			try {
				Certificate[] certchain = ks.getCertificateChain(ks.getCertificateAlias(cert));
				if(certchain != null) {
					DebugInfo.printMessage(3,"TlsKeystore::getCertificateChainEncoded: chain length " + certchain.length);
					chain = new byte[certchain.length][];
					for(int i=0; i<certchain.length; i++) {
						chain[i] = certchain[i].getEncoded();
					}
				}
				else {
					DebugInfo.printMessage(3,"TlsKeystore::getCertificateChainEncoded: return null chain");
					chain = new byte[1][];
					chain[0] = cert.getEncoded();
				}
			} catch (Exception e) {
				DebugInfo.printMessage(1,"===Error! TlsKeystore::getCertificateChain: Exception");
				e.printStackTrace();
			}
		}
		return chain;
	}
}	
